/*==============================================================================
 *
 * (#)DataIO.java
 * Created 2003/10/01
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.main;

//****** imports
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.List;
import java.net.*;

import javax.swing.*;
import javax.swing.border.*;

// dom4j is an easy to use, open source library for working
// with XML, XPath and XSLT on the Java platform using the
// Java Collections Framework and with full support for DOM,
// SAX and JAXP.
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.Attribute;
import org.xml.sax.InputSource;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;
import org.dom4j.io.SAXReader;

// MySQL
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

// Notice, do not import com.mysql.jdbc.* 
// or you will have problems!

import com.miami.romeo.config.*;
import com.miami.romeo.xmame.*;

/**
 * DataIO.<br>
 * This class saves, retrieves or modifies the list of games.
 *
 * @author       $Author: Stefano Bolli
 * @version      $Revision: 1.13 $Date: 2003/12/23 18:25:25 $
 *
 * @since 0.1.3pre2
 */
public class DataIO {
  // Class to read and save configurations
  String config_name;
  File config_file;
  Config config = null;

  // This class dialogs with xmame
  Xmame xmame   = null;

  Utils utils = new Utils();

  UpdateDialog updateDialog = null;

  public JDialog dialog = null;

  // List of games
  private String listOfGamesPath;
  private File listOfGamesFile;
  
  // Vectors containing rom data
  Vector dataRows   = new Vector();
  Vector vector     = new Vector();
  // The rows to be shown in the JTables
  Object[] datarows = null;
  Object[] row      = null;

  // String buffer used to save data in a text file
  StringBuffer bufferLine;

  // Buffered writer to save the list of games in text mode
  BufferedWriter bw = null;

  public static String gameDocRoot           = "Games";
  public static String gameDocEntry          = "Entry";
  public static String gameDocAvailability   = "Available";
  public static String gameDocDescription    = "Description";
  public static String gameDocName           = "Name";
  public static String gameDocProductionYear = "Year";
  public static String gameDocManufacturer   = "Manufacturer";
  public static String gameDocWorkingStatus  = "Working";
  public static String gameDocCloneOf        = "CloneOf";
  public static String gameDocFavorite       = "Favorite";
  public static String gameDocDriver         = "Driver";
  public static String gameDocCategory       = "Category";
  public static String gameDocSample         = "Sample";
  public static String gameDocPlayed         = "Played";
  public static String gameDocPath           = "Path";
  public static String gameDocIcon           = "Icon";

  // Data to populate table rows
  String[] cell = new String[14];

  // XMLWriter takes a DOM4J tree and formats it to a
  // stream as XML. It can also take SAX events too so 
  // can be used by SAX clients as this object implements 
  // the ContentHandler and LexicalHandler interfaces, 
  // as well. This formatter performs typical document 
  // formatting. The XML declaration and processing 
  // instructions are always on their own lines.
  XMLWriter xmlw = null;

  //~ Instance/static variables .............................................

  // Connection to server, initialized in setUp(), cleaned up in tearDown().
  protected Connection conn = null;

  // ResultSet to be used in I/O operations, not initialized.
  // Cleaned up in tearDown().
  protected ResultSet rs = null;

  // Statement to be used in I/O operations, initialized in setUp().
  // Cleaned up in tearDown().
  protected Statement stmt = null;

  // PreparedStatement to be used in I/O operations, not initialized.
  // Cleaned up in tearDown().
  protected PreparedStatement pstmt = null;

  // JDBC URL
  protected static String dbUrl = null;

  Thread thread = null;

  // The main class
  Romeo romeo = null;

  /**
   * The constructor.
   *
   * @param romeo The main class
   */
  public DataIO(Romeo romeo) {
    this.romeo = romeo;
  }

  /**
   * This method saves a new list of games and returns it in a Vector.
   *
   * @param xmame_path The path to the xmame executable
   * @param list_name  The name of the list
   *
   * @see              #showUpdateDialog()
   * @see              #getDataStoringType()
   * @see              #saveXMLDocument(String, String)
   * @see              #saveSQLTable(String, String, String, boolean)
   * @see              #saveTextFile(String, String)
   *
   * @return           The list of games previously saved
   *
   * @throws           InterruptedException
   *
   * @since            0.1.3pre2
   */
  public Vector saveListOfGames(String xmame_path,
                                String list_name) throws InterruptedException {
    Vector data = new Vector();


    showUpdateDialog();

    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    String xmame_execs = config.get("xmame.executable_file");

    String[] xmame_file = utils.parseValues(xmame_execs, ';');

    String list = "list";

    for (int i = 0; i < xmame_file.length; i++) {
      if (xmame_file[i].equals(xmame_path)) {
        list = "list" + i;
        break;
      }
    }

    int data_storing_type = getDataStoringType();

    switch (data_storing_type) {
      case 0:
        data = saveXMLDocument(xmame_path, list_name);
        break;
      case 1:
        data = saveSQLTable(xmame_path, list_name, list,
                     listOfGamesExists(xmame_path, list_name));
        break;
      case 2:
        data = saveTextFile(xmame_path, list_name);
        break;
      default:
        data = saveXMLDocument(xmame_path, list_name);
        break;
    }

    return data;/*getListOfGames(xmame_path, list_name);*/
  }

  /**
   * This method creates a JDialog to show the saving process.
   *
   * @see com.miami.romeo.main.UpdateDialog#getDialog()
   */
  public void showUpdateDialog() {
    updateDialog = new UpdateDialog(romeo.getFrame(),
                                    utils.getString("UpdateDialog.title"),
                                    false);

    dialog = updateDialog.getDialog();

    dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

    // Create a window listener and add it to the dialog box
    WindowListener wl = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        // Verify that the buffered writer is not writing the list of games
	if (bw == null) {
          dialog.setVisible(false);
	  dialog.dispose();
	  dialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
	}
        else
          return;
      }
    };

    dialog.addWindowListener(wl);
  }



  /**
   * This method returns true if the list of games exists, false otherwise.
   *
   * @param xmame_path The path to xmame
   * @param list_name  The name of the list
   *
   * @see              #XMLDocumentExists(String)
   * @see              #SQLTableExists(String, String)
   * @see              #textFileExists(String)
   *
   * @return           <code>True</code> if the list of games exists,
   *                   <code>false</code> otherwise
   *
   * @since            0.1.3pre2
   */
  public boolean listOfGamesExists(String xmame_path, String list_name) {
    boolean listExists;

    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    String table_created = config.get("mysql.table_created");

    String xmame_execs = config.get("xmame.executable_file");

    String[] xmame_file = utils.parseValues(xmame_execs, ';');

    String list = "list";

    for (int i = 0; i < xmame_file.length; i++) {
      if (xmame_file[i].equals(xmame_path)) {
        list = "list" + i;
        break;
      }
    }

    int data_storing_type = getDataStoringType();

    switch (data_storing_type) {
      case 0:
        listExists = XMLDocumentExists(list_name);
        break;
      case 1:
        listExists = SQLTableExists(list, table_created);
        break;
      case 2:
        listExists = textFileExists(list_name);
        break;
      default:
        listExists = XMLDocumentExists(list_name);
        break;
    }

    return listExists;
  }

  /**
   * This method updates the list of games.
   *
   * @param vector     The Vector containing rom data
   * @param xmame_path The path to the xmame executable
   * @param list_name  The name of the list
   *
   * @see              #getDataStoringType()
   * @see              #updateXMLDocument(Vector, String, String)
   * @see              #updateSQLTable(Vector, String, String)
   * @see              #updateTextFile(Vector, String, String)
   *
   * @since                   0.1.3pre2
   */
  public void updateListOfGames(Vector vector, String xmame_path, String list_name) {
    int data_storing_type = getDataStoringType();

    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    String xmame_execs = config.get("xmame.executable_file");

    String[] xmame_file = utils.parseValues(xmame_execs, ';');

    String list = "list";

    for (int i = 0; i < xmame_file.length; i++) {
      if (xmame_file[i].equals(xmame_path)) {
        list = "list" + i;
        break;
      }
    }

    switch (data_storing_type) {
      case 0:
        updateXMLDocument(vector, xmame_path, list_name);
        break;
      case 1:
        updateSQLTable(vector, xmame_path, list);
        break;
      case 2:
        updateTextFile(vector, xmame_path, list_name);
        break;
      default:
        updateXMLDocument(vector, xmame_path, list_name);
        break;
    }
  }

  /**
   * This method calls specific data retrieving void.
   *
   * @param xmame_path The path to xmame
   * @param list_name  The list name
   *
   * @see              #getXMLData(String)
   * @see              #getSQLData(String)
   * @see              #getTextData(String)
   *
   * @return           A <code>Vector</code> containing rom data
   *
   * @since            0.1.3pre2
   */
  public Vector getListOfGames(String xmame_path, String list_name) {
    vector.clear();

    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    String xmame_execs = config.get("xmame.executable_file");

    String[] xmame_file = utils.parseValues(xmame_execs, ';');

    String list = "list";

    for (int i = 0; i < xmame_file.length; i++) {
      if (xmame_file[i].equals(xmame_path)) {
        list = "list" + i;
        break;
      }
    }

    int data_storing_type = getDataStoringType();

    switch (data_storing_type) {
      case 0:
        vector = getXMLData(list_name);
        break;
      case 1:
        vector = getSQLData(list);
        break;
      case 2:
        vector = getTextData(list_name);
        break;
      default:
        vector = getXMLData(list_name);
        break;
    }

    return vector;
  }

  /**
   * This method returns the data storing type: 0 to use xml document (default)
   *                                            1 to use sql table
   *                                            2 to use text file
   *
   * @return Data storing type
   *
   * @since  0.1.3pre2
   */
  public int getDataStoringType() {
    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    return config.getInt("data_storing_type");
  }

  //*******************************************************************
  //******************************* XML *******************************
  //*******************************************************************

  /**
   * This method returns true if the XML Document exists, false otherwise.
   *
   * @param list_name The list name
   *
   * @return          <code>True</code> if the XML Document exists,
   *                  <code>false</code> otherwise
   *
   * @since 0.1.3pre2
   */
  protected boolean XMLDocumentExists(String list_name) {
    // File where rom data will be saved in
    listOfGamesPath = System.getProperty("user.home") + File.separator +
	              ".romeo4xmame" + File.separator + list_name + ".xml";
    listOfGamesFile = new File(listOfGamesPath);

    if (listOfGamesFile.exists())
      return true;
    else
      return false;
  }

  /**
   * This method saves rom data into a xml document and returns a vector.
   *
   * @param xmame_path The path to xmame
   * @param list_name  The name of the list
   *
   * @return           A <code>vector</code> containing rom data
   *
   * @see              #getData(String, String)
   *
   * @since 0.1.3pre2
   */
  protected Vector saveXMLDocument(final String xmame_path,
                                   final String list_name) {
    vector.clear();

    // Let' s use a thread not to freeze Romeo
    thread = new Thread() {
      public void run() {
        // Get data to save
        vector = getData(xmame_path, list_name);

        if (vector.size() > 0) {
	  // Set progress bar maximum value
          updateDialog.setProgressBarMaximum(updateDialog.pBar2, vector.size() + 1);

	  // updateDialog.message to the user
	  updateDialog.message(updateDialog.saving, utils.getString("UpdateDialog.creating_list"));

	  // DocumentHelper is a collection of helper methods
	  // for using DOM4J.
	  DocumentHelper dh = new DocumentHelper();

          // Document defines an XML Document.
	  Document listOfGames = dh.createDocument(dh.createElement(gameDocRoot));

	  // Element interface defines an XML element. An element can
	  // have declared gameNamepaces, attributes, child nodes and
	  // textual content.
	  Element root = listOfGames.getRootElement();

          for (int i = 0; i < vector.size(); i++) {
            row = (Object[])vector.elementAt(i);

            Element entry = dh.createElement(gameDocEntry);
            root.add(entry);

            Element availability = dh.createElement(gameDocAvailability);
	    // Set the text data of this node
	    availability.setText((String) row[0]);
	    entry.add(availability);

	    Element description = dh.createElement(gameDocDescription);
	    description.setText((String) row[1]);
 	    entry.add(description);

	    Element name = dh.createElement(gameDocName);
	    name.setText((String) row[2]);
	    entry.add(name);

	    Element year = dh.createElement(gameDocProductionYear);
	    year.setText((String) row[3]);
	    entry.add(year);

	    Element manufacturer = dh.createElement(gameDocManufacturer);
	    manufacturer.setText((String) row[4]);
	    entry.add(manufacturer);

	    Element working = dh.createElement(gameDocWorkingStatus);
	    working.setText((String) row[5]);
	    entry.add(working);

	    Element cloneof = dh.createElement(gameDocCloneOf);
	    cloneof.setText((String) row[6]);
	    entry.add(cloneof);

	    Element fav = dh.createElement(gameDocFavorite);
	    fav.setText((String) row[7]);
	    entry.add(fav);

	    Element driver = dh.createElement(gameDocDriver);
	    driver.setText((String) row[8]);
	    entry.add(driver);

	    Element category = dh.createElement(gameDocCategory);
	    category.setText((String) row[9]);
	    entry.add(category);

	    Element sam = dh.createElement(gameDocSample);
	    sam.setText((String) row[10]);
	    entry.add(sam);

	    Element pl = dh.createElement(gameDocPlayed);
	    pl.setText((String) row[11]);
	    entry.add(pl);

            Element pt = dh.createElement(gameDocPath);
	    pt.setText((String) row[12]);
	    entry.add(pt);

            Element iconPath = dh.createElement(gameDocIcon);
	    iconPath.setText((String) row[13]);
	    entry.add(iconPath);

	    updateDialog.setProgressBarValue(updateDialog.pBar2, i);
          }

          // An OutputFormat object can be used to define how whitespace
	  // is handled when printing and allows various configuration
	  // options, such as to allow suppression of the XML declaration,
	  // the encoding declaration or whether empty documents are collapsed.
	  OutputFormat of = new OutputFormat("\t", true);

          try {
	    updateDialog.message(updateDialog.saving,
                                 utils.getString("UpdateDialog.saving_list"));

            // File where rom data will be saved in
            listOfGamesPath = System.getProperty("user.home") + File.separator +
	                      ".romeo4xmame" + File.separator + list_name + ".xml";
            listOfGamesFile = new File(listOfGamesPath);

            xmlw = new XMLWriter(new FileOutputStream(listOfGamesFile), of);
	    xmlw.write(listOfGames);
	    updateDialog.setProgressBarValue(updateDialog.pBar2, vector.size() + 1);
            xmlw.close();
	    xmlw = null;
            dialog = updateDialog.getDialog();
	    dialog.setVisible(false);
	    dialog = null;
          }
          catch (Exception e) {
            System.out.println(e.getMessage());
          }
        }
	else {
          dialog = updateDialog.getDialog();
          dialog.setVisible(false);
	  dialog = null;
        }
      }
    };

    thread.start();

    try {
      // Wait for the operation is terminated
      thread.join();
    }
    catch(InterruptedException ie) {
      System.out.println(ie.getMessage());
    }

    return vector;
  }

  /**
   * This method saves rom data into a xml document and returns a vector.
   *
   * @param vector     The <code>Vector</code> containing rom data
   * @param xmame_path The path to xmame
   * @param list_name  The name of the list
   *
   * @return           A <code>vector</code> containing rom data
   *
   * @since 0.1.3pre2
   */
  protected void updateXMLDocument(final Vector vector,
                                   final String xmame_path,
                                   final String list_name) {
    // Let' s use a thread not to freeze Romeo
    thread = new Thread() {
      public void run() {
        if (vector.size() > 0) {
	  // DocumentHelper is a collection of helper methods
	  // for using DOM4J.
	  DocumentHelper dh = new DocumentHelper();

          // Document defines an XML Document.
	  Document listOfGames = dh.createDocument(dh.createElement(gameDocRoot));

	  // Element interface defines an XML element. An element can
	  // have declared gameNamepaces, attributes, child nodes and
	  // textual content.
	  Element root = listOfGames.getRootElement();

          for (int i = 0; i < vector.size(); i++) {
            row = (Object[])vector.elementAt(i);

            Element entry = dh.createElement(gameDocEntry);
            root.add(entry);

            Element availability = dh.createElement(gameDocAvailability);
	    // Set the text data of this node
	    availability.setText((String) row[0]);
	    entry.add(availability);

	    Element description = dh.createElement(gameDocDescription);
	    description.setText((String) row[1]);
 	    entry.add(description);

	    Element name = dh.createElement(gameDocName);
	    name.setText((String) row[2]);
	    entry.add(name);

	    Element year = dh.createElement(gameDocProductionYear);
	    year.setText((String) row[3]);
	    entry.add(year);

	    Element manufacturer = dh.createElement(gameDocManufacturer);
	    manufacturer.setText((String) row[4]);
	    entry.add(manufacturer);

	    Element working = dh.createElement(gameDocWorkingStatus);
	    working.setText((String) row[5]);
	    entry.add(working);

	    Element cloneof = dh.createElement(gameDocCloneOf);
	    cloneof.setText((String) row[6]);
	    entry.add(cloneof);

	    Element fav = dh.createElement(gameDocFavorite);
	    fav.setText((String) row[7]);
	    entry.add(fav);

	    Element driver = dh.createElement(gameDocDriver);
	    driver.setText((String) row[8]);
	    entry.add(driver);

	    Element category = dh.createElement(gameDocCategory);
	    category.setText((String) row[9]);
	    entry.add(category);

	    Element sam = dh.createElement(gameDocSample);
	    sam.setText((String) row[10]);
	    entry.add(sam);

	    Element pl = dh.createElement(gameDocPlayed);
	    pl.setText((String) row[11]);
	    entry.add(pl);

            Element pt = dh.createElement(gameDocPath);
	    pt.setText((String) row[12]);
	    entry.add(pt);

            Element iconPath = dh.createElement(gameDocIcon);
	    iconPath.setText((String) row[13]);
	    entry.add(iconPath);
          }

          // An OutputFormat object can be used to define how whitespace
	  // is handled when printing and allows various configuration
	  // options, such as to allow suppression of the XML declaration,
	  // the encoding declaration or whether empty documents are collapsed.
	  OutputFormat of = new OutputFormat("\t", true);

          try {
            // File where rom data will be saved in
            listOfGamesPath = System.getProperty("user.home") + File.separator +
	                      ".romeo4xmame" + File.separator + list_name + ".xml";
            listOfGamesFile = new File(listOfGamesPath);

            xmlw = new XMLWriter(new FileOutputStream(listOfGamesFile), of);
	    xmlw.write(listOfGames);
            xmlw.close();
	    xmlw = null;
          }
          catch (Exception e) {
            System.out.println(e.getMessage());
          }
        }
      }
    };

    thread.start();
  }

  /**
   * This method returns a vector containing data taken from a XML Document
   *
   * @param list_name The name of the list of games
   *
   * @return          A <code>vector</code> containing rom data
   *
   * @since           0.1.3pre2
   */
  protected Vector getXMLData(String list_name) {
    vector.clear();

    try {
      listOfGamesPath = System.getProperty("user.home") + File.separator +
	                ".romeo4xmame" + File.separator + list_name + ".xml";

      // Get the XML document
      Document log = getXMLDoc(new URL("file:" + listOfGamesPath));

      Element root = log.getRootElement();

      if (root.elements().isEmpty() == false) {
        Iterator lit = root.elementIterator(gameDocEntry);

        while (lit.hasNext()) {
          Element e = (Element)lit.next();

          Iterator ge = e.elementIterator(gameDocAvailability);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[0] = tmp.getText();
          }

          ge = e.elementIterator(gameDocDescription);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[1] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocName);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[2] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocProductionYear);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[3] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocManufacturer);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[4] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocWorkingStatus);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[5] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocCloneOf);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[6] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocFavorite);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[7] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocDriver);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[8] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocCategory);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[9] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocSample);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[10] = tmp.getText();
          }

	  ge = e.elementIterator(gameDocPlayed);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[11] = tmp.getText();
          }

          ge = e.elementIterator(gameDocPath);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[12] = tmp.getText();
          }

          ge = e.elementIterator(gameDocIcon);
          if (ge != null) {
            Element tmp = (Element)ge.next();
            cell[13] = tmp.getText();
          }

	  row = new Object[]{cell[0], cell[1], cell[2],
	                     cell[3], cell[4], cell[5],
			     cell[6], cell[7], cell[8],
			     cell[9], cell[10], cell[11],
                             cell[12], cell[13]};

          vector.addElement(row);
        }
      }
    }
    catch ( Exception e ) {
      System.err.println(e.toString());
    }

    return vector;
  }

  /**
   * This method returns the list of games document.
   *
   * @param url The list of games
   *
   * @return    <code>Document</code>
   */
  public Document getXMLDoc(URL url) throws DocumentException {
    SAXReader reader = new SAXReader();
    Document document = reader.read(url);
    return document;
  }

  /**
   * We have to walk a large XML document tree (the list of
   * games) then for performance we have to use a fast looping
   * method which avoids the cost of creating an Iterator object
   * for each loop.
   *
   * @param document The document we want to walk
   *
   * @see            #listOfGamesWalk(Document, Element)
   */
  public void listOfGamesWalk(Document document) {
    listOfGamesWalk(document, document.getRootElement());
  }

  /**
   * We have to walk a large XML document tree (the list of
   * games) then for performance we have to use a fast looping
   * method which avoids the cost of creating an Iterator object
   * for each loop.
   *
   * @param document The document we want to walk
   * @param element The element to find
   */
  public void listOfGamesWalk(Document document, Element element) {
    for (int i = 0, size = element.nodeCount(); i < size; i++) {
      Node node = element.node(i);

      if (node instanceof Element) {
        List tgtDocNodeAttrs = ((Element)node).attributes();

        List srcDocNodeAttrs = ((Element)document.selectSingleNode(
                               ((Element)node).getPath())).attributes();

        ((Element)node).setAttributes(srcDocNodeAttrs);

        for (int a = 0; i < tgtDocNodeAttrs.size(); i++) {
          Attribute tmp = (Attribute)tgtDocNodeAttrs.get(a);
          System.out.println(tmp.toString());
          tmp.detach();
          ((Element)node).add(tmp);
        }

        listOfGamesWalk(document, (Element) node);
      }
    }
  }

  //*******************************************************************
  //******************************* SQL *******************************
  //*******************************************************************

  /**
   * This method returns true if the SQL table exists, false otherwise.
   *
   * @param list_name     The list name
   * @param table_created This String contains the names of the tables created
   *
   * @return          <code>True</code> if the SQL table exists,
   *                  <code>false</code> otherwise
   *
   * @since 0.1.3pre2
   */
  protected boolean SQLTableExists(String list_name, String table_created) {
    boolean listExists = false;

    if (table_created.length() > 0) {
      String[] list = utils.parseValues(table_created, ';');

      for (int i = 0; i < list.length; i++) {
        if (list[i].equals(list_name)) {
          listExists = true;
          break;
        }
      }
    }

    return listExists;
  }

  /**
   * This method saves rom data into a MySQL' s table and returns a vector.
   *
   * @param xmame_path The path to xmame
   * @param list_name  The list name used by other data storing types
   * @param list       The list name
   * @param listExists True if the list exists, false otherwise
   *
   * @see              #setUp()
   * @see              #tearDown()
   *
   * @return           A <code>vector</code> containing rom data
   *
   * @since            0.1.3pre2
   */
  protected Vector saveSQLTable(final String xmame_path,
                                final String list_name,
                                final String list,
                                final boolean listExists) {
    vector.clear();

    thread = new Thread() {
      public void run() {
        // Get data to save
        vector = getData(xmame_path, list_name);

	if (vector.size() > 0) {
          // Set progress bar maximum value
          updateDialog.setProgressBarMaximum(updateDialog.pBar2, vector.size());

	  try {
	    setUp();

            if (listExists)
              stmt.executeUpdate("DROP TABLE " + list + ";");

            createTable(list);

            String insert = "INSERT INTO " + list + "(" +
                            gameDocAvailability + "," + gameDocDescription + "," +
                            gameDocName + "," + gameDocProductionYear + "," +
                            gameDocManufacturer + "," + gameDocWorkingStatus + "," +
                            gameDocCloneOf + "," + gameDocFavorite + "," +
                            gameDocDriver + "," + gameDocCategory + "," +                                                                              gameDocSample + "," + gameDocPlayed +
                            "," + gameDocPath + "," + gameDocIcon +
                            ") VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

            pstmt = conn.prepareStatement(insert);

            updateDialog.message(updateDialog.saving,
                                 utils.getString("UpdateDialog.saving_list"));

	    for (int i = 0; i < vector.size(); i++) {
              row = (Object[])vector.elementAt(i);

              pstmt.setString(1, (String)row[0]);
              pstmt.setString(2, (String)row[1]);
              pstmt.setString(3, (String)row[2]);
              pstmt.setString(4, (String)row[3]);
              pstmt.setString(5, (String)row[4]);
              pstmt.setString(6, (String)row[5]);
              pstmt.setString(7, (String)row[6]);
              pstmt.setString(8, (String)row[7]);
              pstmt.setString(9, (String)row[8]);
              pstmt.setString(10, (String)row[9]);
              pstmt.setString(11, (String)row[10]);
              pstmt.setString(12, (String)row[11]);
              pstmt.setString(13, (String)row[12]);
              pstmt.setString(14, (String)row[13]);
              pstmt.executeUpdate();

              updateDialog.setProgressBarValue(updateDialog.pBar2, i);
	    }

            tearDown();
            dialog = updateDialog.getDialog();
            dialog.setVisible(false);
	    dialog = null;
	  }
          catch (SQLException ex) {
            // handle any errors
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());
            dialog = updateDialog.getDialog();
            dialog.setVisible(false);
	    dialog = null;
          }
	  catch (Exception e) {
            // handle any errors
            System.out.println("Exception: " + e.getMessage());
            dialog = updateDialog.getDialog();
            dialog.setVisible(false);
	    dialog = null;
          }
        }
      }
    };

    thread.start();

    try {
      // Wait for the operation is terminated
      thread.join();
    }
    catch(InterruptedException ie) {
      System.out.println(ie.getMessage());
    }

    return vector;
  }

  /**
   * This method saves rom data into a MySQL' s table and returns a vector.
   *
   * @param vector     The vector conatining the data to save
   * @param xmame_path The path to xmame
   * @param list_name  The list name
   *
   * @see              #setUp()
   * @see              #tearDown()
   *
   * @return           A <code>vector</code> containing rom data
   *
   * @since            0.1.3pre2
   */
  protected void updateSQLTable(final Vector vector,
                                final String xmame_path,
                                final String list_name) {
    thread = new Thread() {
      public void run() {
	if (vector.size() > 0) {
	  try {
	    setUp();

            stmt.executeUpdate("DROP TABLE " + list_name + ";");

            createTable(list_name);

            String replace = "INSERT INTO " + list_name + "(" +
                             gameDocAvailability + "," + gameDocDescription + "," +
                             gameDocName + "," + gameDocProductionYear + "," +
                             gameDocManufacturer + "," + gameDocWorkingStatus + "," +
                             gameDocCloneOf + "," + gameDocFavorite + "," +
                             gameDocDriver + "," + gameDocCategory + "," +                                                                              gameDocSample + "," + gameDocPlayed + "," +
                             gameDocPath + "," + gameDocIcon +
                             ") VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

            pstmt = conn.prepareStatement(replace);

	    for (int i = 0; i < vector.size(); i++) {
              row = (Object[])vector.elementAt(i);

              pstmt.setString(1, (String)row[0]);
              pstmt.setString(2, (String)row[1]);
              pstmt.setString(3, (String)row[2]);
              pstmt.setString(4, (String)row[3]);
              pstmt.setString(5, (String)row[4]);
              pstmt.setString(6, (String)row[5]);
              pstmt.setString(7, (String)row[6]);
              pstmt.setString(8, (String)row[7]);
              pstmt.setString(9, (String)row[8]);
              pstmt.setString(10, (String)row[9]);
              pstmt.setString(11, (String)row[10]);
              pstmt.setString(12, (String)row[11]);
              pstmt.setString(13, (String)row[12]);
              pstmt.setString(14, (String)row[13]);
              pstmt.executeUpdate();
	    }

            tearDown();
	  }
          catch (SQLException ex) {
            // handle any errors
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());
          }
	  catch (Exception e) {
            // handle any errors
            System.out.println("Exception: " + e.getMessage());
          }
        }
      }
    };

    thread.start();
  }

  /**
   * This method retrieves the list of game from a MySQL' s table.
   *
   * @param  list_name The list name
   *
   * @see    #setUp()
   * @see    #tearDown()
   *
   * @return A <code>vector</code> containing game data
   *
   * @since  0.1.3pre2
   */
  protected Vector getSQLData(String list_name) {
    vector.clear();

    try {
      setUp();

      rs = stmt.executeQuery("SELECT * FROM " + list_name + ";");

      while(rs.next()) {
	cell[0]  = rs.getString(gameDocAvailability);
        cell[1]  = rs.getString(gameDocDescription);
        cell[2]  = rs.getString(gameDocName);
	cell[3]  = rs.getString(gameDocProductionYear);
        cell[4]  = rs.getString(gameDocManufacturer);
        cell[5]  = rs.getString(gameDocWorkingStatus);
        cell[6]  = rs.getString(gameDocCloneOf);
        cell[7]  = rs.getString(gameDocFavorite);
	cell[8]  = rs.getString(gameDocDriver);
	cell[9]  = rs.getString(gameDocCategory);
	cell[10] = rs.getString(gameDocSample);
	cell[11] = rs.getString(gameDocPlayed);
        cell[12] = rs.getString(gameDocPath);
        cell[13] = rs.getString(gameDocIcon);

        row = new Object[]{cell[0], cell[1], cell[2],
	                   cell[3], cell[4], cell[5],
			   cell[6], cell[7], cell[8],
			   cell[9], cell[10], cell[11],
                           cell[12], cell[13]};

        // Store the row in the vector.
	vector.addElement(row);
      }

      tearDown();
    }
    catch (SQLException ex) {
      // handle any errors
      System.out.println("SQLException: " + ex.getMessage());
      System.out.println("SQLState: " + ex.getSQLState());
      System.out.println("VendorError: " + ex.getErrorCode());
    }
    catch (Exception e) {
      // handle any errors
      System.out.println("Exception: " + e.getMessage());
    }

    return vector;
  }

  /**
   * This method creates resources used by getSQLData() & saveSQLTable()
   *
   * @throws <code>Exception</code> if an error occurs
   */
  protected void setUp() throws Exception {
    // Call Config to read the configuration file
    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    String mysql_user = config.get("mysql.user");
    String mysql_passwd = config.get("mysql.passwd");
    String mysql_hostname = config.get("mysql.hostname");

    dbUrl = "jdbc:mysql://" + mysql_hostname + "/romeo?user=" + mysql_user +
            "&password=" + mysql_passwd;

    Class.forName("com.mysql.jdbc.Driver").newInstance();
    this.conn = DriverManager.getConnection(dbUrl);
    this.stmt = conn.createStatement();
  }

  /**
   * This method creates a table.
   *
   * @param list_name The list name
   *
   * @throws          <code>Exception</code> if an error occurs
   *
   * @since           0.1.3pre2
   */
  protected void createTable(String list_name) throws Exception {
    String createDB = "CREATE TABLE IF NOT EXISTS " + list_name +
                      " (ID INT NOT NULL AUTO_INCREMENT PRIMARY KEY, " +
                      gameDocAvailability + " VARCHAR(3) NOT NULL, " +
                      gameDocDescription + " VARCHAR(150) NOT NULL, " +
                      gameDocName + " VARCHAR(100) NOT NULL, " +
                      gameDocProductionYear + " VARCHAR(4) NOT NULL, " +
                      gameDocManufacturer + " VARCHAR(100) NOT NULL, " +
                      gameDocWorkingStatus + " VARCHAR(3) NOT NULL, " +
                      gameDocCloneOf + " VARCHAR(100) NOT NULL, " +
                      gameDocFavorite + " VARCHAR(3) NOT NULL, " +
                      gameDocDriver + " VARCHAR(100) NOT NULL, " +
		      gameDocCategory + " VARCHAR(150) NOT NULL, " +
		      gameDocSample + " VARCHAR(100) NOT NULL, " +
                      gameDocPlayed + " VARCHAR(100) NOT NULL, " +
                      gameDocPath + " VARCHAR(255) NOT NULL, " +
                      gameDocIcon + " VARCHAR(255) NOT NULL);";

    stmt.executeUpdate(createDB);

    // Call Config to read the configuration file
    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    String table_created = config.get("mysql.table_created");

    if (table_created.length() > 0) {
      if(!SQLTableExists(list_name, table_created))
        config.add("mysql.table_created", table_created + ";" + list_name);
    }
    else
        config.add("mysql.table_created", list_name);

    config.save();
  }

  /**
   * This method destroys resources created during the I/O operations.
   *
   * @throws <code>Exception</code> DOCUMENT ME!
   */
  public void tearDown() throws Exception {
    if (this.rs != null) {
      try {
        this.rs.close();
      } catch (SQLException SQLE) {
        ;
      }
    }

    if (this.stmt != null) {
      try {
        this.stmt.close();
      } catch (SQLException SQLE) {
        ;
      }
    }

    if (this.pstmt != null) {
      try {
        this.pstmt.close();
      } catch (SQLException SQLE) {
        ;
      }
    }

    if (this.conn != null) {
      try {
        this.conn.close();
      } catch (SQLException SQLE) {
        ;
      }
    }
  }

  //*******************************************************************
  //****************************** Text *******************************
  //*******************************************************************

  /**
   * This method returns true if the text file exists, false otherwise.
   *
   * @param list_name The list name
   *
   * @return          <code>True</code> if the text file exists,
   *                  <code>false</code> otherwise
   *
   * @since 0.1.3pre2
   */
  protected boolean textFileExists(String list_name) {
    // File where rom data will be saved in
    listOfGamesPath = System.getProperty("user.home") + File.separator +
	              ".romeo4xmame" + File.separator + list_name + ".txt";
    listOfGamesFile = new File(listOfGamesPath);

    if (listOfGamesFile.exists())
      return true;
    else
      return false;
  }

  /**
   * This method saves rom data into a text file.
   *
   * @param xmame_path The path to xmame
   * @param list_name  The name of the list
   *
   * @return           A <code>vector</code> containing game data
   *
   * @since            0.1.3pre2
   */
  protected Vector saveTextFile(final String xmame_path,
                                final String list_name) {
    vector.clear();

    thread = new Thread() {
      public void run() {
        // Get data to save
        vector = getData(xmame_path, list_name);

        if (vector.size() > 0) {
	  // Set progress bar maximum value
          updateDialog.setProgressBarMaximum(updateDialog.pBar2, vector.size());

	  try {
	    // File where roms data will be saved in
            listOfGamesPath = System.getProperty("user.home") +
	                      File.separator + ".romeo4xmame" +
			      File.separator + list_name + ".txt";
            listOfGamesFile = new File(listOfGamesPath);

	    bw = new BufferedWriter(new FileWriter(listOfGamesFile));

	    String headline  = "AVAILABLE DESCRIPTION                        " +
	                       "                                             " +
			       "                     NAME                    " +
			       "       YEAR   MANUFACTURER                   " +
			       "                              WORKING CLONE OF" +
			       "                       FAVORITE  DRIVER      " +
			       "    CATEGORY               SAMPLE            " +
			       "   PLAYED PATH                              " +
                               "ICON\n";
	    String separator = "--------- -----------------------------------" +
	                       "---------------------------------------------" +
			       "-------------------- ------------------------" +
			       "------ ------ -------------------------------" +
			       "----------------------------- ------- -------" +
			       "----------------------- --------- -----------" +
			       "---- ---------------------- -----------------" +
			       "--- ------ --------------------------------- " +
                               "-------------------------------------------\n";

	    bw.write(headline);
	    bw.write(separator);

            updateDialog.message(updateDialog.saving,
                                 utils.getString("UpdateDialog.saving_list"));

	    for (int i = 0; i < vector.size(); i++) {
              row = (Object[])vector.elementAt(i);

	      // Create a string buffer
	      bufferLine = new StringBuffer(350);

	      for (int j = 0; j < 500; j++)
	        bufferLine.insert(j, " ");

              // Insert data
	      bufferLine.insert(0,   (String) row[0]);
	      bufferLine.insert(10,  (String) row[1]);
	      bufferLine.insert(111, (String) row[2]);
	      bufferLine.insert(142, (String) row[3]);
	      bufferLine.insert(149, (String) row[4]);
	      bufferLine.insert(210, (String) row[5]);
	      bufferLine.insert(218, (String) row[6]);
              bufferLine.insert(249, (String) row[7]);
              bufferLine.insert(259, (String) row[8]);
  	      bufferLine.insert(275, (String) row[9]);
	      bufferLine.insert(298, (String) row[10]);
	      bufferLine.insert(319, (String) row[11]);
              bufferLine.insert(325, (String) row[12]);
              bufferLine.insert(358, (String) row[13]);

	      // Write the file
	      bw.write(bufferLine.toString() + "\n");

              updateDialog.setProgressBarValue(updateDialog.pBar2, i);
	    }

	    // Close buffered writer
	    bw.flush();
	    bw.close();
	    bw = null;
            dialog = updateDialog.getDialog();
            dialog.setVisible(false);
	    dialog = null;
	  }
	  catch (IOException ioe) {
	    System.err.println(ioe.getMessage());
	  }
        }
	else {
          dialog = updateDialog.getDialog();
          dialog.setVisible(false);
          dialog = null;
        }
      }
    };

    thread.start();

    try {
      // Wait for the operation is terminated
      thread.join();
    }
    catch(InterruptedException ie) {
      System.out.println(ie.getMessage());
    }

    return vector;
  }

  /**
   * This method updates the text file.
   *
   * @param vector     The vector conatining the data to save
   * @param xmame_path The path to xmame
   * @param list_name  The name of the list
   *
   * @return           A <code>vector</code> containing game data
   *
   * @since            0.1.3pre2
   */
  protected void updateTextFile(final Vector vector,
                                final String xmame_path,
                                final String list_name) {
    thread = new Thread() {
      public void run() {
        if (vector.size() > 0) {
	  try {
	    // File where roms data will be saved in
            listOfGamesPath = System.getProperty("user.home") +
	                      File.separator + ".romeo4xmame" +
			      File.separator + list_name + ".txt";
            listOfGamesFile = new File(listOfGamesPath);

	    bw = new BufferedWriter(new FileWriter(listOfGamesFile));

	    String headline  = "AVAILABLE DESCRIPTION                        " +
	                       "                                             " +
			       "                     NAME                    " +
			       "       YEAR   MANUFACTURER                   " +
			       "                              WORKING CLONE OF" +
			       "                       FAVORITE  DRIVER      " +
			       "    CATEGORY               SAMPLE            " +
			       "   PLAYED PATH                              " +
                               "ICON\n";
	    String separator = "--------- -----------------------------------" +
	                       "---------------------------------------------" +
			       "-------------------- ------------------------" +
			       "------ ------ -------------------------------" +
			       "----------------------------- ------- -------" +
			       "----------------------- --------- -----------" +
			       "---- ---------------------- -----------------" +
			       "--- ------ --------------------------------- " +
                               "-------------------------------------------\n";

	    bw.write(headline);
	    bw.write(separator);

	    for (int i = 0; i < vector.size(); i++) {
              row = (Object[])vector.elementAt(i);

	      // Create a string buffer
	      bufferLine = new StringBuffer(350);

	      for (int j = 0; j < 500; j++)
	        bufferLine.insert(j, " ");

              // Insert data
	      bufferLine.insert(0,   (String) row[0]);
	      bufferLine.insert(10,  (String) row[1]);
	      bufferLine.insert(111, (String) row[2]);
	      bufferLine.insert(142, (String) row[3]);
	      bufferLine.insert(149, (String) row[4]);
	      bufferLine.insert(210, (String) row[5]);
	      bufferLine.insert(218, (String) row[6]);
              bufferLine.insert(249, (String) row[7]);
              bufferLine.insert(259, (String) row[8]);
  	      bufferLine.insert(275, (String) row[9]);
	      bufferLine.insert(298, (String) row[10]);
	      bufferLine.insert(319, (String) row[11]);
              bufferLine.insert(325, (String) row[12]);
              bufferLine.insert(358, (String) row[13]);

	      // Write the file
	      bw.write(bufferLine.toString() + "\n");
	    }

	    // Close buffered writer
	    bw.flush();
	    bw.close();
	    bw = null;
	  }
	  catch (IOException ioe) {
	    System.err.println(ioe.getMessage());
	  }
        }
      }
    };

    thread.start();
  }

  /**
   * This method returns a vector containing data taken from a text file.
   *
   * @param list_name The name of the list of games
   *
   * @since           0.1.3pre2
   *
   * @return          <code>Vector</code>
   */
  protected Vector getTextData(String list_name) {
    vector.clear();

    try {
      listOfGamesPath = System.getProperty("user.home") +
                        File.separator + ".romeo4xmame" +
			File.separator + list_name + ".txt";

      listOfGamesFile = new File(listOfGamesPath);

      BufferedReader br = new BufferedReader(new FileReader(listOfGamesFile));

      String line;
      while ((line = br.readLine()) != null) {
        if ((!line.startsWith("AVAILABLE")) && (!line.startsWith("----"))) {
	  cell[0]  = line.substring(0, 9).trim();
          cell[1]  = line.substring(10, 110).trim();
          cell[2]  = line.substring(111, 141).trim();
          cell[3]  = line.substring(142, 148).trim();
          cell[4]  = line.substring(149, 209).trim();
          cell[5]  = line.substring(210, 217).trim();
          cell[6]  = line.substring(218, 249).trim();
          cell[7]  = line.substring(249, 259).trim();
	  cell[8]  = line.substring(259, 275).trim();
	  cell[9]  = line.substring(275, 298).trim();
	  cell[10] = line.substring(298, 319).trim();
	  cell[11] = line.substring(319, 325).trim();
          cell[12] = line.substring(325, 358).trim();
          cell[13] = line.substring(358).trim();

          for (int j = 0; j < cell.length; j++) {
            row = new Object[]{cell[0], cell[1], cell[2],
	                       cell[3], cell[4], cell[5],
			       cell[6], cell[7], cell[8],
			       cell[9], cell[10], cell[11],
                               cell[12], cell[13]};
          }

          // Store the row in the vector.
	  vector.addElement(row);
	}
      }

      br.close();
    }
    catch (IOException e) {
      System.err.println(e.getMessage());
    }

    return vector;
  }

  /**
   * This method returns a vector containing rom data.
   *
   * @param xmame_path The path to the xmame executable
   * @param list_name  The path to the list of games
   *
   * @return           <code>Vector</code> of rom data
   */
  protected Vector getData(final String xmame_path, final String list_name) {
    dataRows.clear();

    // updateDialog.message to the user
    updateDialog.message(updateDialog.updating,
                         utils.getString("UpdateDialog.retrieving_data"));

    updateDialog.setProgressBarMaximum(updateDialog.pBar, 17);

    xmame = new Xmame(romeo);

    //******* Get data from xmame executable
    // List supported games with full description
    String[] listFull       = xmame.listFull(xmame_path);
    updateDialog.setProgressBarValue(updateDialog.pBar, 1);
    // Get game names
    String[] gameName       = xmame.getRomNames(listFull);
    //System.out.println("nomi: "  + gameName.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 2);
    Vector v = getAvailableRoms();
    // Get available roms
    String[] availableGame  = getListOfAvailableRoms(v, gameName);
    //System.out.println("disponibili: " + availableGame.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 3);
    // Get roms descriptions
    String[] description    = xmame.getRomDescriptions(listFull);
    //System.out.println("descrizioni: " + description.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 4);
    // List supported games with manufacturer and year
    String[] listGames      = xmame.listGames(xmame_path);
    updateDialog.setProgressBarValue(updateDialog.pBar, 5);
    // Get production year
    String[] year           = xmame.getRomYears(listGames);
    //System.out.println("anno: " + year.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 6);
    // Get manufacturers
    String[] manufacturer   = xmame.getRomManufacturers(listGames);
    //System.out.println("produttori: " + manufacturer.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 7);
    // Get working roms
    String[] workingRom     = xmame.getWorkingRoms(xmame_path);
    //System.out.println("funzionanti: " + workingRom.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 8);
    // List clones
    String[] listClones     = xmame.listClones(xmame_path);
    updateDialog.setProgressBarValue(updateDialog.pBar, 9);
    // Get 'roms clones of'
    String[] clone          = xmame.getClones(gameName, listClones);
    //System.out.println("cloni: " + clone.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 10);
    // Get favorite roms
    String[] favoriteRom    = xmame.getFavoriteRoms(gameName, list_name);
    //System.out.println("favoriti: " + favoriteRom.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 11);
    // Get source files
    String[] sourceFile     = xmame.getSourceFiles(xmame_path);
    //System.out.println("source: " + sourceFile.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 12);
    // Get roms categories
    String[] romCategory    = getRomCategories(gameName);
    //System.out.println("categorie: " + romCategory.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 13);
    // Get roms using audio samples
    String[] audioSample    = xmame.getRomsUsingSamples(xmame_path, gameName);
    //System.out.println("sample: " + audioSample.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 14);
    // Get number of played matches for each game
    String[] number         = xmame.getNumberOfPlayedMatches(gameName, list_name);
    //System.out.println("number: " + number.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 15);
    // Get rom paths
    String[] romPath        = getRomPaths(v, gameName);
    //System.out.println("percorsi: " + romPath.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 16);
    // Get paths to available mamu' s icons
    String[] icon           = getMamuIcons(gameName, clone);
    //System.out.println("icone: " + icon.length);
    updateDialog.setProgressBarValue(updateDialog.pBar, 17);

    // Check wheter all arrays have same length
    if ((availableGame.length == description.length) &&
        (description.length == gameName.length) &&
        (gameName.length == year.length) &&
	(year.length == manufacturer.length) &&
	(manufacturer.length == workingRom.length) &&
        (workingRom.length == clone.length) &&
	(clone.length == favoriteRom.length) &&
	(favoriteRom.length == sourceFile.length) &&
	(sourceFile.length == romCategory.length) &&
  	(romCategory.length == audioSample.length) &&
	(audioSample.length == number.length) &&
        (number.length == romPath.length) &&
        romPath.length == icon.length) {

      for (int i = 0; i < gameName.length; i++) {
	for (int j = 0; j < 14; j++) {
          datarows = new Object[]{availableGame[i], description[i],
	                          gameName[i], year[i], manufacturer[i],
                                  workingRom[i], clone[i], favoriteRom[i],
	 		          sourceFile[i], romCategory[i],
			          audioSample[i], number[i], romPath[i],
                                  icon[i]};
        }

        // Store the row in the vector.
	dataRows.addElement(datarows);
      }

      updateDialog.message(updateDialog.updating,
                           utils.getString("UpdateDialog.retrieving_data_done"));
    }
    else {
      System.out.println("Could not create the list of games: " +
	                 "arrays do not have same length!");
    }

    return dataRows;
  }

  /**
   * This method returns an array of strings with the paths to Mamu' s icons.
   *
   * @param gameName The game names
   * @param clone    The list of clones
   *
   * @return         Paths to Mamu' s icons
   */
  public String[] getMamuIcons( String[] gameName, String[] clone) {
    ArrayList iconList = new ArrayList();
    String iconPath = new String();

    String prefix = System.getProperty("user.home") + File.separator +
                    ".romeo4xmame" + File.separator + "MamuIcons";

    File mamuDir = new File(prefix);

    if (mamuDir.exists()) {
      String[] path = scanMamuDir(prefix);

      for (int i = 0; i < gameName.length; i++) {
        iconPath = "-";
        String icon = gameName[i] + ".png";

        for (int j = 0; j < path.length; j++) {
          if (clone[i].equals("-")) {
            if (icon.equals(path[j])) {
              iconPath = prefix + File.separator + icon;
              break;
            }
          }
          else {
            String originalGameIcon = clone[i] + ".png";
            if (originalGameIcon.equals(path[j])) {
              iconPath = prefix + File.separator + originalGameIcon;
              break;
            }
          }
        }
        
        iconList.add(iconPath);
      }
    }
    else {
      for (int i = 0; i < gameName.length; i++) {
        iconList.add("-");
      }
    }

    return (String[])iconList.toArray(new String[0]);
  }

  /**
   * This method scans Mamu dir and retrieves paths to available icons.
   */
  public String[] scanMamuDir(String prefix) {
    // Mamu directory
    File mamuDir = new File(prefix + File.separator);
    // Filter for mamu' s icons
    MamuFilter filter = new MamuFilter();

    // Paths to available Mamu' s icons
    String[] iconList = mamuDir.list(filter);

    return iconList;
  }

  /**
   * Filter for icons.
   */
  class MamuFilter implements FilenameFilter {
    public boolean accept(File dir, String name) {
      boolean acceptFile = false;
      if (name.endsWith(".png"))
        acceptFile = true;
      return acceptFile;
    }
  }

  /**
   * This method counts how many roms are available.
   *
   * @return The <code>number (int)</code> of available roms
   *
   * @see    #getAvailableRoms()
   *
   * @since  0.1.3pre2
   */
  public int countRoms(String xmame_path) {
    Vector availableRoms = getAvailableRoms();

    return availableRoms.size();
  }

  /**
   * This method returns an array with the same length of the game names. It' s filled
   * with a "No" if a game is not available, with "Yes" otherwise.
   *
   * @param rom      A Vector filled with the available games and their absolute path
   * @param gameName The list of game names
   *
   * @return         The <code>list</code> of game availability
   *
   * @since          0.1.3pre2
   */
  public String[] getListOfAvailableRoms(Vector rom, String[] gameName) {
    ArrayList availableRomList = new ArrayList();
    String result = new String();
    
    for (int i = 0; i < gameName.length; i++) {
      result = "No";

      for (int j = 0; j < rom.size(); j++) {
        Object[] vector_row = (Object[])rom.elementAt(j);

        int idx_zip = ((String)vector_row[0]).indexOf(".zip");

        if (gameName[i].equals(((String)vector_row[0]).substring(0, idx_zip))) {
          result = "Yes";
          break;
        }
      }

      availableRomList.add(result);
    }

    return (String[])availableRomList.toArray(new String[0]);
  }

  /**
   * This method returns the list of absolute paths for each game.
   *
   * @param rom      A Vector filled with the available games and their absolute path
   * @param gameName The list of game names
   *
   * @return         The <code>list</code> of absolute path for the each game
   *
   * @since          0.1.3pre2
   */
  public String[] getRomPaths(Vector rom, String[] gameName) {
    ArrayList romPathList = new ArrayList();
    String result = new String();

    for (int i = 0; i < gameName.length; i++) {
      result = "-";

      for (int j = 0; j < rom.size(); j++) {
        Object[] vector_row = (Object[])rom.elementAt(j);

        int idx_zip = ((String)vector_row[0]).indexOf(".zip");

        if (gameName[i].equals(((String)vector_row[0]).substring(0, idx_zip))) {
          result = (String)vector_row[1];
          break;
        }
      }

      romPathList.add(result);
    }

    return (String[])romPathList.toArray(new String[0]);
  }

  /**
   * This method retrieves available roms.
   *
   * @return      The <code>list</code> of available games
   *
   * @see         #getRomDirs()
   */
  public Vector getAvailableRoms() {
    Vector rom_list = new Vector();

    boolean already_present;

    File[] romDir = getRomDirs();

    RomFilter romFilter = new RomFilter();

    xmame = new Xmame(romeo);

    for (int i = 0; i < romDir.length; i++) {
      already_present = false;
      String[] listOfAvailableRoms = romDir[i].list(romFilter);

      for (int j = 0; j < listOfAvailableRoms.length; j++) {
        // Now verify if the rom has been yet inserted into the list
        for (int z = 0; z < rom_list.size(); z++) {
          Object[] list = (Object[])rom_list.elementAt(z);
          if (listOfAvailableRoms[j].equals((String)list[0])) {
            already_present = true;
            break;
          }
        }

        if (!already_present)
          rom_list.addElement(new Object[] {listOfAvailableRoms[j],
                                            romDir[i].getAbsolutePath()});
      }
    }

    // Call Config to save the number of available roms found
    config_name = System.getProperty("user.home") +
                  File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    config.add("xmame.available_roms", rom_list.size());
    config.save();

    return rom_list;
  }

  /**
   * This method puts the rom dirs set by the user into an array of files.
   *
   * @see    #getRomDirPaths()
   *
   * @return The <code>list</code> of rom directories
   *
   * @since  0.1.3pre2
   */
  public File[] getRomDirs() {
    String[] romDirPath = getRomDirPaths();

    int numberOfRomDirs = romDirPath.length;

    File[] romDir = new File[numberOfRomDirs];

    for (int i = 0; i < numberOfRomDirs; i++) {
      romDir[i] = new File(romDirPath[i]);
    }

    return romDir;
  }

  /**
   * This method gets the paths to the rom directories and then put them
   * into an array of strings.
   *
   * @return The <code>list</code> of rom dir paths
   *
   * @see    com.miami.romeo.config.Utils#parseValues(String, char)
   *
   * @since 0.1.3pre2
   */
  public String[] getRomDirPaths() {
    // Call Config to read the configuration file
    config_name = System.getProperty("user.home") +
	          File.separator + ".romeo4xmame" +
		  File.separator + "Romeo.ini";
    config_file = new File(config_name);
    config      = new Config(config_file);

    // Get the line containing rom dirs
    String rom_dir = config.get("xmame.rom_dir");

    // Ask Utils to parse the line
    String[] dir = utils.parseValues(rom_dir, ';');

    return dir;
  }

  /**
   * Filter for roms.
   */
  class RomFilter implements FilenameFilter {
    public boolean accept(File dir, String name) {
      boolean acceptFile = false;
      if (name.endsWith(".zip"))
        acceptFile = true;
      return acceptFile;
    }
  }

  /**
   * This method retrieves a list of rom categories from
   * catver.ini.
   *
   * @param gameName The games
   *
   * @return         The list of categories
   *
   * @see            com.miami.romeo.xmame.Xmame#getRomNames(String[])
   * @see            #getInfoFromCatver(String)
   *
   * @since          0.1.3pre1
   */
  public String[] getRomCategories(String[] gameName) {
    String romCategory = "[Unknown]";
    String catverPath = config.get("xmame.catver.ini_path");
    String[] catverLines = getInfoFromCatver(catverPath);
    ArrayList listOfCategories = new ArrayList();

    for (int i = 0; i < gameName.length; i++) {
      for (int j = 0; j < catverLines.length; j++) {
        // Parse line
        int idx_e = catverLines[j].indexOf('=');

	if (idx_e > 0) {
	  if (gameName[i].equals(catverLines[j].substring(0, idx_e))) {
            romCategory = catverLines[j].substring(idx_e + 1);
	    break;
	  }
        }
      }

      listOfCategories.add(romCategory);
    }

    return (String[])listOfCategories.toArray(new String[0]);
  }

  /**
   * This method retrieves data from catver.ini
   *
   * @param catverPath The path to catver.ini
   *
   * @return           An <code>array</code> of data strings from catver.ini
   */
  public String[] getInfoFromCatver(String catverPath) {
    ArrayList lines = new ArrayList();

    try {
      File catver = new File(catverPath);

      if (catver.exists()) {
        BufferedReader br = new BufferedReader(new FileReader(catver));

        String line;

        while ((line = br.readLine()) != null) {
          // Do not add unuseful things
          if ((!line.startsWith(";")) && (!line.equals("[Category]")) &&
	      (!line.equals("")))
            lines.add(line.trim());

	    if (line.equals(
	          "; ****************** Version Informaion *************") ||
		line.equals(
		  "; ****************** Version Information *************") ||
		line.equals("[VerAdded]"))
	    break;
          }

        br.close();
      }
    }
    catch (IOException e) {
      System.err.println(e);
    }

    return (String[])lines.toArray(new String[0]);
  }
}
