
/*==============================================================================
 *
 * (#)Options.java
 * created      2003/03/26
 * version      $Revision: 1.13 $Date: 2003/12/31 16:14:03 $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.main;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import com.miami.romeo.config.*;
import com.miami.romeo.xmame.*;
import com.miami.romeo.internet.*;
import com.miami.romeo.settings.*;
import com.miami.romeo.help.*;
import com.miami.romeo.main.table.IndicatorCellRenderer;


/**
 * Panel containing game options.
 */
public class Options extends JPanel {
  Romeo romeo = null;
  Utils utils = new Utils();
  Config config = null;
  Xmame xmame = null;
  IndicatorCellRenderer iRenderer = null;

  JPanel panel                = null;
  JPanel northPanel           = null;
  JPanel centerPanel          = null;
  JLabel label                = null;
  ChangeListener sListener    = null;
  
  // Parameters to pass to MAME
  String gameOptions     = new String();
  String gameName        = new String();
  String mamePath        = new String();
  String romPath         = new String();

  boolean iconifyWindows = false;

  // If checked Romeo will run a game with default options
  JCheckBox defaultBox        = null;

  // Buttons to run a game and to save game options
  JButton runRom              = null;
  JButton saveOptions         = null;

  /* MAME core sound options */
  public JComboBox dspBox     = null;
  String dsp                  = "";
  JCheckBox timerBox          = null;
  String timer                = " -timer";
  public JComboBox mixerBox   = null;
  String mixer                = "";
  JCheckBox soundBox          = null;
  String sound                = " -sound";
  JCheckBox samplesBox        = null;
  String samples              = " -samples";
  JCheckBox fakesoundBox      = null;
  String fakesound            = "";
  JComboBox samplefreqBox     = null;
  String samplefreq           = "";
  JTextField bufsizeField     = null;
  String bufsize              = "";
  JSlider volumeBox           = null;
  String volume               = " -volume 0";
  JTextField audiodeviceField = null;
  String audiodevice          = "";
  JTextField mixerdeviceField = null;
  String mixerdevice          = "";
  JCheckBox resamplefilterBox = null;
  String resamplefilter       = " -resamplefilter";

  JPanel audioPanel           = null;

  /* Video options */
  JCheckBox autoframeskipBox  = null;
  String autoframeskip        = " -autoframeskip";
  JComboBox frameskipTypeBox  = null;
  String frameskipType        = " -frameskipper 0";
  ButtonGroup frameskipGroup  = null;
  JTextField frameskipField   = null;
  String frameskip            = " -frameskip 8";
  JCheckBox windowBox         = null;
  String window               = "";
  JCheckBox scanlinesBox      = null;
  String scanlines            = " -noscanlines";
  JCheckBox throttleBox       = null;
  String throttle             = " -throttle";
  JComboBox effectBox         = null;
  String effect               = "";
  JCheckBox autodoubleBox     = null;
  String autodouble           = " -noautodouble";
  JList gammaBox              = null;
  String gamma                = " -gamma 0.5";
  JSlider brightnessBox       = null;
  String brightness           = " -brightness 1";
  JTextField geometryField    = null;
  String geometry             = "";

  /* Display Related */
  JCheckBox rotateBox   = null;
  String rotate         = " -nonorotate";
  JCheckBox autororBox  = null;
  String autoror        = " -noautoror";
  JCheckBox autorolBox  = null;
  String autorol        = " -noautorol";
  JCheckBox rorBox      = null;
  String ror            = " -noror";
  JCheckBox rolBox      = null;
  String rol            = " -norol";
  JCheckBox flipxBox    = null;
  String flipx          = " -noflipx";
  JCheckBox flipyBox    = null;
  String flipy          = " -noflipy";
  
  /* Artwork Related */
  JCheckBox artworkBox         = null;
  String artwork               = " -artwork";
  JCheckBox backdropBox        = null;
  String backdrop              = " -backdrop";
  JCheckBox overlayBox         = null;
  String overlay               = " -overlay";
  JCheckBox bezelBox           = null;
  String bezel                 = " -bezel";
  JCheckBox cropBox            = null;
  String crop                  = " -noartcrop";
  JTextField aresolutionField  = null;
  String artworkresolution     = " -artres 0";

  /* Vector Games Related */
  JTextField vectorField  = null;
  JSlider beamBox         = null;
  String beam             = " -beam 1";
  JSlider flickerBox      = null;
  String flicker          = " -flicker 0";
  JSlider intensityBox    = null;
  String intensity        = " -intensity 1.5";
  JCheckBox antialiasBox  = null;
  String antialias        = " -antialias";
  JCheckBox tlucencyBox   = null;
  String translucency     = " -translucency";

  /* Input device options */
  JCheckBox mouseBox          = null;
  String mouse                = " -nomouse";
  JComboBox joytypeBox        = null;
  String joytype              = " -joytype 0";
  JCheckBox joystickBox       = null;
  String joystick             = "";
  JCheckBox steadykeyBox      = null;
  String steadykey            = "";
  JCheckBox keyboardledsBox   = null;
  String keyboardleds         = "";
  JComboBox ctrlrBox          = null;
  String ctrlr                = "";
  JTextField joydevnameField  = null;
  String joydevname           = "";
  JCheckBox analogstickBox    = null;
  String analogstick          = "";
  JCheckBox hotrodBox         = null;
  String hotrod               = "";
  JCheckBox hotrodseBox       = null;
  String hotrodse             = "";
  JCheckBox usbpspadBox       = null;
  String usbpspad             = "";
  JCheckBox rapidfireBox      = null;
  String rapidfire            = "";

  /**
   * Constructor.
   *
   * @param romeo The caller 
   */
  public Options(Romeo romeo) {
    this.romeo = romeo;

    setLayout(new BorderLayout());

    add(createNorthPanel(), BorderLayout.NORTH);

    String OS = System.getProperty("os.name").toLowerCase();
    // System.out.println(OS);
    if (OS.indexOf("windows") > -1) {
      this.window = " -nowindow";

      centerPanel = new JPanel(new BorderLayout());
      centerPanel.add(createWindowsAudioPanel(), BorderLayout.WEST);
      
      JPanel videoPanel = new JPanel();
      videoPanel.setLayout(new BorderLayout());
      videoPanel.setBorder(BorderFactory.createTitledBorder(
                                          utils.getString("Options.video")));
      videoPanel.add(createWindowsVideoPanel(), BorderLayout.WEST);

      JPanel videoPanel2 = new JPanel();
      videoPanel2.setLayout(new GridLayout(2, 1));
      videoPanel2.add(createArtworkOptionsPanel());
      videoPanel2.add(createDisplayRotationPanel());
      videoPanel.add(videoPanel2, BorderLayout.CENTER);

      JPanel videoPanel3 = new JPanel();
      videoPanel3.setLayout(new GridLayout(2, 1));
      videoPanel3.add(createWindowsInputPanel());
      videoPanel3.add(createVectorGamesOptionsPanel());
      videoPanel.add(videoPanel3, BorderLayout.EAST);

      centerPanel.add(videoPanel, BorderLayout.CENTER);

      add(centerPanel, BorderLayout.CENTER);
    }
    else {
      centerPanel = new JPanel(new BorderLayout());
      centerPanel.add(createLinuxAudioPanel(), BorderLayout.WEST);
      
      JPanel videoPanel = new JPanel();
      videoPanel.setLayout(new BorderLayout());
      videoPanel.setBorder(BorderFactory.createTitledBorder(
                                          utils.getString("Options.video")));
      videoPanel.add(createLinuxVideoPanel(), BorderLayout.WEST);

      JPanel videoPanel2 = new JPanel();
      videoPanel2.setLayout(new GridLayout(2, 1));
      videoPanel2.add(createArtworkOptionsPanel());
      videoPanel2.add(createDisplayRotationPanel());
      videoPanel.add(videoPanel2, BorderLayout.CENTER);

      JPanel videoPanel3 = new JPanel();
      videoPanel3.setLayout(new GridLayout(2, 1));
      videoPanel3.add(createLinuxInputPanel());
      videoPanel3.add(createVectorGamesOptionsPanel());
      videoPanel.add(videoPanel3, BorderLayout.EAST);

      centerPanel.add(videoPanel, BorderLayout.CENTER);
      
      add(centerPanel, BorderLayout.CENTER);
    }

    //resetOptions();
  }

  /**
   * This method returns a panel.
   */
  public JPanel createNorthPanel() {
    northPanel = new JPanel();
    northPanel.setLayout(new GridLayout(1, 2));

    defaultBox = new JCheckBox();
    defaultBox.setText(utils.getString("Options.jcheckbox_default"));
    defaultBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
	  resetOptions();
          enableComponents(true);
	  saveOptions.setEnabled(false);
        }
        else {
	  resetOptions();
          enableComponents(false);
	  saveOptions.setEnabled(true);

	  File gameConfig = new File(System.getProperty("user.home") +
                                     File.separator + ".romeo4xmame" +
                                     File.separator + "Games" + File.separator +
                                     romeo.games.gameName + "rc");

	  if (gameConfig.exists())
	    loadYourOptions();
	  else
	    resetOptions();
        }
      }
    });

    panel = new JPanel();
    panel.setLayout(new BorderLayout());
    panel.setBorder(BorderFactory.createTitledBorder(
                                          utils.getString("Options.default")));
    panel.add(defaultBox, BorderLayout.WEST);

    northPanel.add(panel);

    panel = new JPanel();
    panel.setLayout(null);
    panel.setBorder(BorderFactory.createTitledBorder(
                      utils.getString("Options.your_options")));

    runRom = new JButton();
    runRom.setIcon(utils.createImageIcon24x24("Exec.png"));
    runRom.setToolTipText(utils.getString("Options.play"));
    runRom.setBounds(10, 15, 30, 30);
    runRom.addActionListener(new PlayGameAction(this));
    panel.add(runRom);

    saveOptions = new JButton();
    saveOptions.setIcon(utils.createImageIcon24x24("Save.png"));
    saveOptions.setToolTipText(utils.getString("Options.save"));
    saveOptions.setBounds(45, 15, 30, 30);
    saveOptions.addActionListener(new SaveYourOptionsAction(this));
    panel.add(saveOptions);

    northPanel.add(panel);

    return northPanel;
  }

  /**
   * This method returns a panel containing audio options for the
   * Windows command line.
   */
  public JPanel createWindowsAudioPanel() {    
    soundBox = new JCheckBox();
    soundBox.setText(utils.getString("Options.sound"));
    soundBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          sound = " -sound";
        else
          sound = " -nosound";
      }
    });

    samplesBox = new JCheckBox();
    samplesBox.setText(utils.getString("Options.samples"));
    samplesBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          samples = " -samples";
        else
          samples = " -nosamples";
      }
    });

    JLabel vl = new JLabel(utils.getString("Options.volume") + " 0");

    volumeBox = new JSlider(-32, 0, 0);
    volumeBox.setPreferredSize(new Dimension(200, 40));
    volumeBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    volumeBox.setPaintTicks(true);
    volumeBox.setMajorTickSpacing(1);
    sListener = new SliderListener(vl);
    volumeBox.addChangeListener(sListener);

    String[] rate = {" 11025", " 22050", " 44100", " 48000"};

    samplefreqBox = new JComboBox(rate);
    samplefreqBox.setPreferredSize(new Dimension(200, 24));
    samplefreqBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
	samplefreq = " -samplerate " + cb.getSelectedItem().toString();
      }
    });

    resamplefilterBox = new JCheckBox();
    resamplefilterBox.setText("Resample filter");
    resamplefilterBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          resamplefilter = " -resamplefilter";
        else
          resamplefilter = " -noresamplefilter";
      }
    });

    panel = new JPanel();
    panel.setLayout(new GridLayout(17, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                                            utils.getString("Options.audio")));
    panel.add(soundBox);
    panel.add(samplesBox);
    panel.add(new JLabel(utils.getString("Options.samplefreq")));
    panel.add(samplefreqBox);
    panel.add(new JLabel(utils.getString("Options.volume")));
    panel.add(volumeBox);
    panel.add(resamplefilterBox);
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));

    return panel;
  }

  /**
   * This method returns a panel containing audio options for xmame.
   */
  public JPanel createLinuxAudioPanel() {
    xmame = new Xmame(romeo);
    String xmamePath = romeo.games.getXmamePath();
    
    audioPanel = new JPanel();
    audioPanel.setLayout(new BorderLayout());
    audioPanel.setBorder(BorderFactory.createTitledBorder(
                                            utils.getString("Options.audio")));

    dspBox = new JComboBox();
    dspBox.setPreferredSize(new Dimension(200, 24));
    dspBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();

	if (!cb.getSelectedItem().toString().equals("<NOT SET>"))
	  dsp = " -dp " + cb.getSelectedItem().toString().trim();
	else
          dsp = "";
      }
    });

    timerBox = new JCheckBox();
    timerBox.setText(utils.getString("Options.timer"));
    timerBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          timer = " -timer";
        else
          timer = " -notimer";
      }
    });

    mixerBox = new JComboBox();
    mixerBox.setPreferredSize(new Dimension(200, 24));
    mixerBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();

	if (!cb.getSelectedItem().toString().equals("<NOT SET>"))
	  mixer = " -smp " + cb.getSelectedItem().toString().trim();
	else
          mixer = "";
      }
    });

    panel = new JPanel();
    panel.setLayout(new GridLayout(5, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                                utils.getString("Options.digital_audio")));
    panel.add(new JLabel(utils.getString("Options.dsp_label")));
    panel.add(dspBox);
    panel.add(timerBox);
    panel.add(new JLabel(utils.getString("Options.mixer_label")));
    panel.add(mixerBox); 

    audioPanel.add(panel, BorderLayout.NORTH);

    soundBox = new JCheckBox();
    soundBox.setText(utils.getString("Options.sound"));
    soundBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          sound = " -sound";
        else
          sound = " -nosound";
      }
    });

    samplesBox = new JCheckBox();
    samplesBox.setText(utils.getString("Options.samples"));
    samplesBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          samples = " -samples";
        else
          samples = " -nosamples";
      }
    });

    fakesoundBox = new JCheckBox();
    fakesoundBox.setText(utils.getString("Options.fakesound"));
    fakesoundBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          fakesound = " -fakesound";
        else
          fakesound = "";
      }
    });

    JLabel vl = new JLabel(utils.getString("Options.volume") + " 0");

    volumeBox = new JSlider(-32, 0, 0);
    volumeBox.setPreferredSize(new Dimension(200, 40));
    volumeBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    volumeBox.setPaintTicks(true);
    volumeBox.setMajorTickSpacing(1);
    sListener = new SliderListener(vl);
    volumeBox.addChangeListener(sListener);

    String[] rate = {" 11025", " 22050", " 44100", " 48000"};

    samplefreqBox = new JComboBox(rate);
    samplefreqBox.setPreferredSize(new Dimension(200, 24));
    samplefreqBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
	samplefreq = " -samplefreq " + cb.getSelectedItem().toString();
      }
    });

    bufsizeField = new JTextField();
    bufsizeField.addKeyListener(new KeyAdapter() {
      public void keyTyped(KeyEvent e) {
        char c = e.getKeyChar();
        if (!((Character.isDigit(c) ||
            (c == '.') ||
            (c == KeyEvent.VK_BACK_SPACE) ||
            (c == KeyEvent.VK_DELETE)))) {
          getToolkit().beep();
          e.consume();
        }
      }
    });
    bufsizeField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0) {
          try {
            Float.parseFloat(content);
            bufsize = " -bufsize " + content;
          } catch (NumberFormatException nfe) {
            getToolkit().beep();
            textField.setText("");
            textField.requestFocus();
            JOptionPane.showMessageDialog(null,
                                    utils.getString(
				    "Messagebox.error_novalidfloat"),
                                    utils.getString("Messagebox.error_title"),
                                    JOptionPane.ERROR_MESSAGE);
          }
        }
      }
    });

    audiodeviceField = new JTextField();
    audiodeviceField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0)
          audiodevice = " -audiodevice " + content;
      }
    });

    mixerdeviceField = new JTextField();
    mixerdeviceField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0)
          mixerdevice = " -mixerdevice " + content;
      }
    });

    panel = new JPanel();
    panel.setLayout(new GridLayout(14, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.general")));
    panel.add(soundBox);
    panel.add(samplesBox);
    panel.add(fakesoundBox);
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel(utils.getString("Options.samplefreq")));
    panel.add(samplefreqBox);
    panel.add(new JLabel(utils.getString("Options.bufsize")));
    panel.add(bufsizeField);
    panel.add(vl);
    panel.add(volumeBox);
    panel.add(new JLabel(utils.getString("Options.audiodevice")));
    panel.add(audiodeviceField);
    panel.add(new JLabel(utils.getString("Options.mixerdevice")));
    panel.add(mixerdeviceField);

    audioPanel.add(panel, BorderLayout.CENTER);

    return audioPanel;
  }

  /**
   * This method creates a panel containing Windows video options.
   */
  public JPanel createWindowsVideoPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(17, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.video")));

    autoframeskipBox = new JCheckBox();
    autoframeskipBox.setText(utils.getString("Options.autoframeskip"));
    autoframeskipBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          autoframeskip = " -autoframeskip";
        else
          autoframeskip = " -noautoframeskip";
      }
    });

    frameskipField = new JTextField();
    frameskipField.addKeyListener(new KeyAdapter() {
      public void keyTyped(KeyEvent e) {
        char c = e.getKeyChar();
        if (!((Character.isDigit(c) ||
            (c == KeyEvent.VK_BACK_SPACE) ||
            (c == KeyEvent.VK_DELETE)))) {
          getToolkit().beep();
          e.consume();
        }
      }
    });
    frameskipField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0) {
          try {
            Integer.parseInt(content);
	    frameskip = " -frameskip " + content;
          } catch (NumberFormatException nfe) {
            getToolkit().beep();
            textField.setText("");
            textField.requestFocus();
	    JOptionPane.showMessageDialog(null,
                               utils.getString("Messagebox.error_novalidint"),
                               utils.getString("Messagebox.error_title"),
                               JOptionPane.ERROR_MESSAGE);
          }
        }
      }
    });

    windowBox = new JCheckBox();
    windowBox.setText(utils.getString("Options.fullscreen"));
    windowBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          window = " -nowindow";
        else
          window = " -window";
      }
    });

    throttleBox = new JCheckBox();
    throttleBox.setText(utils.getString("Options.throttle"));
    throttleBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          throttle = " -throttle";
        else
          throttle = " -nothrottle";
      }
    });

    scanlinesBox = new JCheckBox();
    scanlinesBox.setText(utils.getString("Options.scanlines"));
    scanlinesBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          throttle = " -scanlines";
        else
          throttle = " -noscanlines";
      }
    });

    String[] effects = {" none", " sharp", " scan25", " scan50",
                        " scan75", " scan75v", " rgb3", " rgb4",
			" rgb4v", " rgb6", " rgb16", " rgbtiny"};

    effectBox = new JComboBox(effects);
    effectBox.setPreferredSize(new Dimension(200, 24));
    effectBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
	effect = " -effect " + cb.getSelectedItem().toString();
      }
    });
    
    final String[] gamma_values = {"0.5", "0.6", "0.7", "0.8", "0.9", "1.0",
                                   "1.1", "1.2", "1.3", "1.4", "1.5", "1.6",
                 	           "1.7", "1.8", "1.9", "2.0"};

    gammaBox = new JList(gamma_values);
    gammaBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // Add a mouse listener to the list
    gammaBox.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        int index = gammaBox.locationToIndex(e.getPoint());

	gamma =" -gamma " + gamma_values[index];
      }
    });
    JScrollPane listScrollPane = new JScrollPane(gammaBox);
    listScrollPane.setPreferredSize(new Dimension(200, 24));

    JLabel btl = new JLabel(utils.getString("Options.brightness") + ": 1.0");

    brightnessBox = new JSlider(5, 20, 10);
    brightnessBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    sListener = new SliderListener(btl);
    brightnessBox.addChangeListener(sListener);

    panel.add(autoframeskipBox);
    panel.add(new JLabel(utils.getString("Options.maxautoframeskip") + ":"));
    panel.add(frameskipField);
    panel.add(windowBox);
    panel.add(throttleBox);
    panel.add(scanlinesBox);
    panel.add(new JLabel(utils.getString("Options.videoeffect") + ":"));
    panel.add(effectBox);
    panel.add(new JLabel("gamma:"));
    panel.add(listScrollPane);
    panel.add(btl);
    panel.add(brightnessBox);
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));
    panel.add(new JLabel("<html>"));

    return panel;
  }

  /**
   * This method creates a panel containing xmame video options.
   */
  public JPanel createLinuxVideoPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(17, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.video")));

    autoframeskipBox = new JCheckBox();
    autoframeskipBox.setText(utils.getString("Options.autoframeskip"));
    autoframeskipBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          autoframeskip = " -autoframeskip";
        else
          autoframeskip = " -noautoframeskip";
      }
    });
    
    String[] types = {" 0 Dos frameskip code",
                      " 1 Enhanced frameskip code"};

    frameskipTypeBox = new JComboBox(types);
    frameskipTypeBox.setPreferredSize(new Dimension(200, 24));
    frameskipTypeBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
	frameskipType = " -frameskipper " + cb.getSelectedIndex();
      }
    });

    frameskipField = new JTextField();
    frameskipField.addKeyListener(new KeyAdapter() {
      public void keyTyped(KeyEvent e) {
        char c = e.getKeyChar();
        if (!((Character.isDigit(c) ||
            (c == KeyEvent.VK_BACK_SPACE) ||
            (c == KeyEvent.VK_DELETE)))) {
          getToolkit().beep();
          e.consume();
        }
      }
    });
    frameskipField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0) {
          try {
            Integer.parseInt(content);
	    frameskip = " -frameskip " + content;
          } catch (NumberFormatException nfe) {
            getToolkit().beep();
            textField.setText("");
            textField.requestFocus();
	    JOptionPane.showMessageDialog(null,
                               utils.getString("Messagebox.error_novalidint"),
                               utils.getString("Messagebox.error_title"),
                               JOptionPane.ERROR_MESSAGE);
          }
        }
      }
    });

    windowBox = new JCheckBox();
    windowBox.setText(utils.getString("Options.fullscreen"));
    windowBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        xmame = new Xmame(romeo);

        String version = xmame.getVersion(romeo.games.getXmamePath());
        String versionName = version.substring(version.indexOf('(') + 1,
                                               version.indexOf(')'));

        if (e.getStateChange() == ItemEvent.SELECTED) {
	  if (versionName.equals("SDL"))
            window = " -fullscreen";
          else if (versionName.equals("x11"))
            window = " -x11 1";
	}
	else {
          if (versionName.equals("SDL"))
            window = " -nofullscreen";
          else if (versionName.equals("x11"))
            window = " -x11 0";
        }
      }
    });

    throttleBox = new JCheckBox();
    throttleBox.setText(utils.getString("Options.throttle"));
    throttleBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          throttle = " -throttle";
        else
          throttle = " -nothrottle";
      }
    });

    scanlinesBox = new JCheckBox();
    scanlinesBox.setText(utils.getString("Options.scanlines"));
    scanlinesBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          throttle = " -scanlines";
        else
          throttle = " -noscanlines";
      }
    });

    String[] effects = {" 0 none (default)", " 1 scale2x (smooth scaling effect)",
                        " 2 scan2 (light scanlines)", " 3 rgbstripe (3x2 rgb vertical stripes)",
			" 4 rgbscan (2x3 rgb horizontal scanlines)", " 5 scan3 (3x3 deluxe scanlines)"};

    effectBox = new JComboBox(effects);
    effectBox.setPreferredSize(new Dimension(200, 24));
    effectBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
	effect = " -effect " + cb.getSelectedIndex();
      }
    });

    final String[] gamma_values = {"0.5", "0.6", "0.7", "0.8", "0.9", "1.0",
                                   "1.1", "1.2", "1.3", "1.4", "1.5", "1.6",
                 	           "1.7", "1.8", "1.9", "2.0"};

    gammaBox = new JList(gamma_values);
    gammaBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // Add a mouse listener to the list
    gammaBox.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        int index = gammaBox.locationToIndex(e.getPoint());

	gamma =" -gamma " + gamma_values[index];
      }
    });
    JScrollPane listScrollPane = new JScrollPane(gammaBox);
    listScrollPane.setPreferredSize(new Dimension(200, 24));

    JLabel btl = new JLabel(utils.getString("Options.brightness") + ": 1.0");

    brightnessBox = new JSlider(5, 20, 10);
    brightnessBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    sListener = new SliderListener(btl);
    brightnessBox.addChangeListener(sListener);
    
    geometryField = new JTextField();
    geometryField.addKeyListener(new KeyAdapter() {
      public void keyTyped(KeyEvent e) {
        char c = e.getKeyChar();
        if (!((Character.isDigit(c) ||
            (c == 'x') ||
            (c == KeyEvent.VK_BACK_SPACE) ||
            (c == KeyEvent.VK_DELETE)))) {
          getToolkit().beep();
          e.consume();
        }
      }
    });
    geometryField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0) {
	  geometry = " -geometry " + content;
        }
	else
          geometry = "";
      }
    });

    autodoubleBox = new JCheckBox();
    autodoubleBox.setText(utils.getString("Options.autodouble"));

    panel.add(autoframeskipBox);
    panel.add(new JLabel(utils.getString("Options.autoframeskip")));
    panel.add(frameskipTypeBox);
    panel.add(new JLabel(utils.getString("Options.maxautoframeskip") + " "));
    panel.add(frameskipField);
    panel.add(windowBox);
    panel.add(throttleBox);
    panel.add(scanlinesBox);
    panel.add(new JLabel(utils.getString("Options.videoeffect") + ":"));
    panel.add(effectBox);
    panel.add(new JLabel("gamma:"));
    panel.add(listScrollPane);
    panel.add(btl);
    panel.add(brightnessBox);
    panel.add(new JLabel(utils.getString("Options.geometry")));
    panel.add(geometryField);
    panel.add(autodoubleBox);

    return panel;
  }

  /**
   * This method creates a panel containing artwork options.
   */
  public JPanel createArtworkOptionsPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(7, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.artwork")));

    artworkBox = new JCheckBox();
    artworkBox.setText(utils.getString("Options.use_artwork"));
    artworkBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          artwork = " -artwork";
        else
          artwork = " -noartwork";
      }
    });

    backdropBox = new JCheckBox();
    backdropBox.setText(utils.getString("Options.use_backdrop_artwork"));
    backdropBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          backdrop = " -backdrop";
        else
          backdrop = " -nobackdrop";
      }
    });

    overlayBox = new JCheckBox();
    overlayBox.setText(utils.getString("Options.use_overlay_artwork"));
    overlayBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          overlay = " -overlay";
        else
          overlay = " -nooverlay";
      }
    });

    bezelBox = new JCheckBox();
    bezelBox.setText(utils.getString("Options.use_bezel_artwork"));
    bezelBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          bezel = " -bezel";
        else
          bezel = " -nobezel";
      }
    });

    cropBox = new JCheckBox();
    cropBox.setText(utils.getString("Options.use_crop_artwork"));
    cropBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          crop = " -artcrop";
        else
          crop = " -noartcrop";
      }
    });

    aresolutionField = new JTextField();
    aresolutionField.addKeyListener(new KeyAdapter() {
      public void keyTyped(KeyEvent e) {
        char c = e.getKeyChar();
        if (!((Character.isDigit(c) ||
            (c == KeyEvent.VK_BACK_SPACE) ||
            (c == KeyEvent.VK_DELETE)))) {
          getToolkit().beep();
          e.consume();
        }
      }
    });
    aresolutionField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0) {
          try {
            Integer.parseInt(content);
	    artworkresolution = " -artres " + content;
          } catch (NumberFormatException nfe) {
            getToolkit().beep();
            textField.setText("");
            textField.requestFocus();
	    JOptionPane.showMessageDialog(null,
                               utils.getString("Messagebox.error_novalidint"),
                               utils.getString("Messagebox.error_title"),
                               JOptionPane.ERROR_MESSAGE);
          }
        }
      }
    });

    panel.add(artworkBox);
    panel.add(backdropBox);
    panel.add(overlayBox);
    panel.add(bezelBox);
    panel.add(new JLabel(utils.getString("Options.artworkresolution") + " "));
    panel.add(aresolutionField);
    panel.add(cropBox);

    return panel;
  }

  /**
   * This method creates a panel containing options to rotate display.
   */
  public JPanel createDisplayRotationPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(7, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.display")));

    rotateBox = new JCheckBox();
    rotateBox.setText(utils.getString("Options.rotate"));
    rotateBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          rotate = " -nonorotate";
        else
          rotate = " -norotate";
      }
    });
    
    autororBox = new JCheckBox();
    autororBox.setText("autoror");
    autororBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          autoror = " -autoror";
        else
          autoror = " -noautoror";
      }
    });

    autorolBox = new JCheckBox();
    autorolBox.setText("autorol");
    autorolBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          autorol = " -autorol";
        else
          autorol = " -noautorol";
      }
    });

    rorBox = new JCheckBox();
    rorBox.setText(utils.getString("Options.rotateright"));
    rorBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          ror = " -ror";
        else
          ror = " -noror";
      }
    });

    rolBox = new JCheckBox();
    rolBox.setText(utils.getString("Options.rotateleft"));
    rolBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          rol = " -rol";
        else
          rol = " -norol";
      }
    });

    flipxBox = new JCheckBox();
    flipxBox.setText(utils.getString("Options.flipx"));
    flipxBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          flipx = " -flipx";
        else
          flipx = " -noflipx";
      }
    });

    flipyBox = new JCheckBox();
    flipyBox.setText(utils.getString("Options.flipy"));
    flipyBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          flipy = " -flipy";
        else
          flipy = " -noflipy";
      }
    });

    panel.add(rotateBox);
    panel.add(autororBox);
    panel.add(autorolBox);
    panel.add(rorBox);
    panel.add(rolBox);
    panel.add(flipxBox);
    panel.add(flipyBox);

    return panel;
  }
  
  /**
   * This method returns a panel containing vector games options.
   */
  public JPanel createVectorGamesOptionsPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(8, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.vector")));
    /*
    vectorres = new JTextField();

    JPanel vectorresPanel = new JPanel();
    vectorresPanel.setLayout(new GridLayout(1, 2));
    vectorresPanel.add(new JLabel(utils.getString("Options.vectorres") + " "));
    vectorresPanel.add(vectorres);
    */
    
    JLabel bl = new JLabel(utils.getString("Options.beam") + " 1.0");

    beamBox = new JSlider(1000, 16000, 1000);
    beamBox.setPreferredSize(new Dimension(200, 40));
    beamBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    //beamBox.setPaintTicks(true);
    //beamBox.setMajorTickSpacing(1);
    sListener = new SliderListener(bl);
    beamBox.addChangeListener(sListener);

    JLabel fl = new JLabel(utils.getString("Options.flicker") + " 0.0");

    flickerBox = new JSlider(0, 100000, 0);
    flickerBox.setPreferredSize(new Dimension(200, 40));
    flickerBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    //flickerBox.setPaintTicks(true);
    //flickerBox.setMajorTickSpacing(1);
    sListener = new SliderListener(fl);
    flickerBox.addChangeListener(sListener);
    
    JLabel il = new JLabel(utils.getString("Options.intensity") + " 1.5");
    
    intensityBox = new JSlider(5, 30, 15);
    intensityBox.setPreferredSize(new Dimension(200, 40));
    intensityBox.putClientProperty("JSlider.isFilled", Boolean.TRUE );
    //intensityBox.setPaintTicks(true);
    //intensityBox.setMajorTickSpacing(1);
    sListener = new SliderListener(il);
    intensityBox.addChangeListener(sListener);
    
    antialiasBox = new JCheckBox();
    antialiasBox.setText(utils.getString("Options.antialias"));
    antialiasBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          antialias = " -antialias";
        else
          antialias = " -noantialias";
      }
    });

    tlucencyBox = new JCheckBox();
    tlucencyBox.setText(utils.getString("Options.translucency"));
    tlucencyBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          translucency = " -translucency";
        else
          translucency = " -notranslucency";
      }
    });

    panel.add(bl);
    panel.add(beamBox);
    panel.add(fl);
    panel.add(flickerBox);
    panel.add(il);
    panel.add(intensityBox);
    panel.add(antialiasBox);
    panel.add(tlucencyBox);

    return panel;
  }

  /**
   * This method returns a panel containing Windows input options.
   */
  public JPanel createWindowsInputPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(5, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.input")));

    mouseBox = new JCheckBox();
    mouseBox.setText("mouse");
    mouseBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          mouse = " -mouse";
        else
          mouse = " -nomouse";
      }
    });
    
    joystickBox = new JCheckBox();
    joystickBox.setText("joystick");
    joystickBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          joystick = " -joystick";
        else
          joystick = " -nojoystick";
      }
    });
    
    steadykeyBox = new JCheckBox();
    steadykeyBox.setText("steady key");
    steadykeyBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          steadykey = " -steadykey";
        else
          steadykey = " -nosteadykey";
      }
    });
    
    keyboardledsBox = new JCheckBox();
    keyboardledsBox.setText("keyboard leds");
    keyboardledsBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          keyboardleds = " -keyboard_leds";
        else
          keyboardleds = " -nokeyboard_leds";
      }
    });
    
    JPanel c = new JPanel(new BorderLayout());
    c.add(new JLabel("Enables support for special controllers:"), BorderLayout.CENTER);

    String[] ctrlrs = {" hotrod", " hotrodse", " slikstik", " xarcade"};
    
    ctrlrBox = new JComboBox(ctrlrs);
    ctrlrBox.setPreferredSize(new Dimension(200, 24));
    ctrlrBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();

	ctrlr = " -ctrlr " + cb.getSelectedItem().toString().trim();
      }
    });
    c.add(ctrlrBox, BorderLayout.SOUTH);

    panel.add(mouseBox);
    panel.add(joystickBox);
    panel.add(steadykeyBox);
    panel.add(keyboardledsBox);
    panel.add(c);

    return panel;
  }
  
  /**
   * This method returns a panel containing xmame input options.
   */
  public JPanel createLinuxInputPanel() {
    panel = new JPanel();
    panel.setLayout(new GridLayout(8, 1));
    panel.setBorder(BorderFactory.createTitledBorder(
                           utils.getString("Options.input")));

    JPanel joytypePanel = new JPanel();
    joytypePanel.setLayout(new BorderLayout());
    joytypePanel.add(new JLabel(utils.getString("Options.joytype") + " "),
                     BorderLayout.WEST);
    
    String[] joytypes = {" 0 No joystick",
                         " 1 i386 style joystick driver",
                         " 2 Fm Town Pad support",
                         " 3 X11 input extension joystick",
                         " 4 new i386 linux 1.x.x joystick driver",
                         " 5 NetBSD USB joystick driver",
                         " 6 PS2-Linux native pad",
                         " 7 SDL joystick driver"};

    joytypeBox = new JComboBox(joytypes);
    joytypeBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();

	joytype = " -joytype " + cb.getSelectedIndex();
      }
    });
    joytypePanel.add(joytypeBox, BorderLayout.CENTER);

    JPanel joydevnamePanel = new JPanel();
    joydevnamePanel.setLayout(new BorderLayout());
    joydevnamePanel.add(new JLabel(utils.getString("Options.joydevname") + " "), 
                        BorderLayout.WEST);
    joydevnameField = new JTextField();
    joydevnameField.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        JTextField textField = (JTextField)e.getSource();
        String content = textField.getText();
        if (content.length() != 0)
          joydevname = " -joydevname " + content;
      }
    });
    joydevnamePanel.add(joydevnameField, BorderLayout.CENTER);
    
    mouseBox = new JCheckBox();
    mouseBox.setText("mouse");
    mouseBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          mouse = " -mouse";
        else
          mouse = " -nomouse";
      }
    });

    analogstickBox = new JCheckBox();
    analogstickBox.setText(utils.getString("Options.analogstick"));
    analogstickBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          analogstick = " -analogstick";
        else
          analogstick = " -noanalogstick";
      }
    });

    hotrodBox = new JCheckBox();
    hotrodBox.setText(utils.getString("Options.hotrod"));
    hotrodBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          hotrod = " -hotrod";
        else
          hotrod = " -nohotrod";
      }
    });

    hotrodseBox = new JCheckBox();
    hotrodseBox.setText(utils.getString("Options.hotrodse"));
    hotrodseBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          hotrodse = " -hotrodse";
        else
          hotrodse = " -nohotrodse";
      }
    });

    usbpspadBox = new JCheckBox();
    usbpspadBox.setText("USB PS Game Pads");
    usbpspadBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          usbpspad = " -usbpspad";
        else
          usbpspad = " -nousbpspad";
      }
    });

    rapidfireBox = new JCheckBox();
    rapidfireBox.setText(utils.getString("Options.rapidfire"));
    rapidfireBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
          rapidfire = " -rapidfire";
        else
          rapidfire = " -norapidfire";
      }
    });
    
    panel.add(joytypePanel);
    panel.add(joydevnamePanel);
    panel.add(mouseBox);
    panel.add(analogstickBox);
    panel.add(hotrodBox);
    panel.add(hotrodseBox);
    panel.add(usbpspadBox);
    panel.add(rapidfireBox);
    
    return panel;
  }
  
  class SliderListener implements ChangeListener {
    JLabel l;

    public SliderListener(JLabel label) {
      l = label;
    }

    public void stateChanged(ChangeEvent e) {
      String s = new String();
      JSlider s1 = (JSlider)e.getSource();
      if (s1 == volumeBox) {
        volume = " -volume " + s1.getValue();
        l.setText(utils.getString("Options.volume") + " " +
                  s1.getValue() + "db");
      }
      else if (s1 == beamBox) {
        float value = (float)(s1.getValue() / 1000);
        beam = " -beam " + value;
        l.setText(utils.getString("Options.beam") + " " +
                  value);
      }
      else if (s1 == flickerBox) {
        float value = (float)(s1.getValue() / 1000);
        flicker = " -flicker " + value;
        l.setText(utils.getString("Options.flicker") + " " +
                  value);
      }
      else if (s1 == intensityBox) {
        if (s1.getValue() < 10)
          s = "0." + s1.getValue();
        else if (s1.getValue() == 10)
          s = "1.0";
        else if (s1.getValue() > 10 && s1.getValue() < 20) {
          String parse = "1." + s1.getValue();
	  s = parse.substring(0, 2) + parse.substring(3);
        }
        else if (s1.getValue() == 20)
          s = "2.0";
	else if (s1.getValue() > 20 & s1.getValue() < 30) {
	  String parse = "2." + s1.getValue();
	  //System.out.println(parse);
          s = parse.substring(0, 2) + parse.substring(3);
	}
	else if (s1.getValue() == 30)
	  s = "3.0";

        intensity = " -intensity " + s;
        l.setText(utils.getString("Options.intensity") + " " + s);
      }
      else if (s1 == brightnessBox) {
        if (s1.getValue() < 10)
          s = "0." + s1.getValue();
        else if (s1.getValue() == 10)
          s = "1.0";
        else if (s1.getValue() > 10 && s1.getValue() < 20) {
          String parse = "1." + s1.getValue();
	  s = parse.substring(0, 2) + parse.substring(3);
        }
        else if (s1.getValue() == 20)
          s = "2.0";

        brightness = " -brightness " + s;
        l.setText(utils.getString("Options.brightness") + ": " + s);
      }
    }
  }

  public void setOptionsComponentsLocked() {
    if (defaultBox.isSelected())
      defaultBox.setSelected(false);
    else
      defaultBox.setSelected(true);

    File gameConfig = new File(System.getProperty("user.home") +
                                     File.separator + ".romeo4xmame" +
                                     File.separator + "Games" + File.separator +
                                     romeo.games.gameName + "rc");

    if (gameConfig.exists())
      defaultBox.setSelected(false);
    else
      defaultBox.setSelected(true);
  }
  
  /**
   * PlayGame Action
   */
  class PlayGameAction extends AbstractAction {
    Options options;
    protected PlayGameAction(Options options) {
      super("PlayGame");
      this.options = options;
    }

    public void actionPerformed(ActionEvent e) {
       runMame();
    }
  }
  
  /**
   * Run Mame and load a selected game.
   *
   * @see #getWindowsOptions()
   * @see #getLinuxOptions()
   * @see #getOtherOptions()
   * @see com.miami.romeo.xmame.Xmame#runMame(String, String)
   */
  public void runMame() {
    xmame = new Xmame(romeo);
    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));

    gameName = romeo.games.gameName;
    mamePath = romeo.games.getXmamePath();
    romPath = " -rompath " + romeo.games.getGamePath(romeo.games.getRomDirs(),
              gameName);
	      
    int n = config.getInt(romeo.games.getNameOfList(mamePath) + ".total_played");
    if (n == -1)
      n = 0;
    int MAX = n + 1;
    config.add(romeo.games.getNameOfList(mamePath) + ".total_played", MAX);
    config.save();

    romeo.games.setNumberOfPlayedMatches(gameName, MAX);

    if (!defaultBox.isSelected()) {
      String OS = System.getProperty("os.name").toLowerCase();
      // System.out.println(OS);
      if (OS.indexOf("windows") > -1) {
        gameOptions = " " + gameName + getWindowsOptions() + getOtherOptions() +
	              romPath;
      }
      else {
        gameOptions = " " + gameName + getLinuxOptions() + getOtherOptions() +
	              romPath;
      }
      
      romeo.games.appendMessage("Info", "Running " + romeo.statusField.getText() +
                                " with options: " + 
				gameOptions.substring(gameOptions.indexOf('-')));
    }
    else {
      gameOptions = " " + gameName + romPath;
      romeo.games.appendMessage("Info", "Running " + romeo.statusField.getText() +
                              " with default options.");
    }
    
    //System.out.println(gameOptions);

    iconifyWindows = config.getBoolean("Iconify_before_playing");

    Thread thread = new Thread() {
      public void run() {
        try {
          if (iconifyWindows) {
            Frame[] frame = romeo.getFrame().getFrames();
            for (int i = 0; i < frame.length; i++) {
              if (frame[i] != null)
                frame[i].setState(Frame.ICONIFIED);
            }
          }

          xmame.runMame(mamePath, gameOptions);
        }
        catch(IOException ioe) {
          System.err.println(ioe.getMessage());
        }
      }
    };

    thread.start();

    try {
      thread.join();

      if (iconifyWindows) {
        Frame[] frame = romeo.getFrame().getFrames();

        for (int i = 0; i < frame.length; i++) {
          if (frame[i] != null)
            frame[i].setState(Frame.NORMAL);
        }
      }
    }
    catch(InterruptedException ie) {
      System.out.println(ie.getMessage());
    }
  }
  
  /**
   * This method enables/disables all components.
   *
   * @param  enable If <code>true</code> enables all components,
   *                else disables them.
   *
   * @since  0.1.3pre3
   */
  public void enableComponents(boolean enable) {
    romeo.setLocked(enable, centerPanel);
  }

  /**
   * This method resets options to their original value.
   *
   * @see    #resetWindowsOptions()
   * @see    #resetLinuxOptions()
   *
   * @since  0.1.3pre3
   */
  public void resetOptions() {
    String OS = System.getProperty("os.name").toLowerCase();
    // System.out.println(OS);
    if (OS.indexOf("windows") > -1) {
      resetWindowsOptions();
    }
    else {
      resetLinuxOptions();
    }
  }
  
  /**
   * This method resets Windows options to their original value.
   *
   * @since  0.1.3pre3
   */
  public void resetWindowsOptions() {
    boolean[] bvalues = {true,
                         true,
			 true,
                         true,
			 false,
			 true,
			 false,
			 true,
			 true,
			 true,
			 true,
			 false,
			 true,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 true,
			 true,
			 true};

    JCheckBox[] bcomp = {soundBox, samplesBox, resamplefilterBox,
                         autoframeskipBox, windowBox, throttleBox,
			 scanlinesBox, artworkBox, backdropBox, overlayBox,
			 bezelBox, cropBox,
			 rotateBox, autororBox, autorolBox,rorBox,
			 rolBox, flipxBox, flipyBox, mouseBox, joystickBox,
			 steadykeyBox, keyboardledsBox, antialiasBox,
			 tlucencyBox};

    for (int i = 0; i < bvalues.length; i++) {
      bcomp[i].setSelected(bvalues[i]);
    }

    int[] ivalues = {2,
                     0,
		     0};

    JComboBox[] icomp = {samplefreqBox,
                         effectBox,
		         ctrlrBox};
			  
    for (int i = 0; i < ivalues.length; i++) {
      icomp[i].setSelectedIndex(ivalues[i]);
    }
    
    gammaBox.setSelectedIndex(1);

    int[] svalues = {0,
                     10,
		     1000,
		     0,
		     15};

    JSlider[] scomp = {volumeBox, brightnessBox, beamBox,
                       flickerBox, intensityBox};

    for (int i = 0; i < svalues.length; i++) {
      scomp[i].setValue(svalues[i]);
    }
    
    String[] tvalues = {"0",
                        "0"};

    JTextField[] tcomp = {frameskipField, aresolutionField};

    for (int i = 0; i < tvalues.length; i++) {
      tcomp[i].setText(tvalues[i]);
    }
  }
  
  /**
   * This method resets XMame options to their original value.
   *
   * @since  0.1.3pre3
   */
  public void resetLinuxOptions() {
    boolean[] bvalues = {true,
                         true,
			 true,
			 false,
                         true,
			 false,
			 false,
			 true,
			 true,
			 true,
			 true,
			 true,
			 true,
			 false,
			 true,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 false,
			 true,
			 true};

    JCheckBox[] bcomp = {timerBox, soundBox,
                         samplesBox, fakesoundBox,
                         autoframeskipBox, autodoubleBox,
			 windowBox, throttleBox,
			 scanlinesBox, artworkBox,
			 backdropBox, overlayBox,
			 bezelBox, cropBox,
			 rotateBox, autororBox,
			 autorolBox,rorBox,
			 rolBox, flipxBox,
			 flipyBox, mouseBox,
			 analogstickBox, hotrodBox,
			 hotrodseBox, usbpspadBox,
			 rapidfireBox, antialiasBox,
			 tlucencyBox};

    for (int i = 0; i < bvalues.length; i++) {
      bcomp[i].setSelected(bvalues[i]);
    }

    int[] ivalues = {0,
                     0,
		     2,
		     1,
                     0,
		     0};

    JComboBox[] icomp = {dspBox, mixerBox, samplefreqBox, frameskipTypeBox,
                         effectBox, joytypeBox};

    for (int i = 0; i < ivalues.length; i++) {
      icomp[i].setSelectedIndex(ivalues[i]);
    }
    
    gammaBox.setSelectedIndex(1);

    int[] svalues = {0,
                     10,
		     1000,
		     0,
		     15};


    JSlider[] scomp = {volumeBox, brightnessBox, beamBox,
                       flickerBox, intensityBox};

    for (int i = 0; i < svalues.length; i++) {
      scomp[i].setValue(svalues[i]);
    }

    String[] tvalues = {"8",
                        "3.000",
			"",
			"",
                        "0",
			"",
			""};

    JTextField[] tcomp = {frameskipField, bufsizeField,
                          audiodeviceField, mixerdeviceField,
			  aresolutionField, geometryField,
                          joydevnameField};

    for (int i = 0; i < tvalues.length; i++) {
      tcomp[i].setText(tvalues[i]);
    }
  }

  /**
   * This method loads user options
   *
   * @see #loadWindowsOptions()
   * @see #loadLinuxOptions()
   */
  public void loadYourOptions () {
    String OS = System.getProperty("os.name").toLowerCase();
    // System.out.println(OS);
    if (OS.indexOf("windows") > -1) {
      loadWindowsOptions();
    }
    else {
      loadLinuxOptions();
    }
  }
  
  /**
   * This method loads Windows options.
   *
   * @since  0.1.3pre3
   */
  public void loadWindowsOptions() {
    File gameConfig = new File(System.getProperty("user.home") + File.separator +
                               ".romeo4xmame" + File.separator +
                               "Games" + File.separator +
                               romeo.games.gameName + "rc");

    Config config = new Config(gameConfig);

    boolean[] bvalues = {config.getBoolean("sound"),
                         config.getBoolean("samples"),
			 config.getBoolean("resamplefilter"),
                         config.getBoolean("autoframeskip"),
			 config.getBoolean("window"),
			 config.getBoolean("throttle"),
			 config.getBoolean("scanlines"),
			 config.getBoolean("artwork"),
			 config.getBoolean("backdrop"),
			 config.getBoolean("overlay"),
			 config.getBoolean("bezel"),
			 config.getBoolean("crop"),
			 config.getBoolean("rotate"),
			 config.getBoolean("autoror"),
			 config.getBoolean("autorol"),
			 config.getBoolean("ror"),
			 config.getBoolean("rol"),
			 config.getBoolean("flipx"),
			 config.getBoolean("flipy"),
			 config.getBoolean("mouse"),
			 config.getBoolean("joystick"),
			 config.getBoolean("steadykey"),
			 config.getBoolean("keyboardleds"),
			 config.getBoolean("antialias"),
			 config.getBoolean("translucency")};

    JCheckBox[] bcomp = {soundBox, samplesBox, resamplefilterBox,
                         autoframeskipBox, windowBox, throttleBox,
			 scanlinesBox, artworkBox, backdropBox, overlayBox,
			 bezelBox, cropBox,
			 rotateBox, autororBox, autorolBox,rorBox,
			 rolBox, flipxBox, flipyBox, mouseBox, joystickBox,
			 steadykeyBox, keyboardledsBox, antialiasBox,
			 tlucencyBox};

    for (int i = 0; i < bvalues.length; i++) {
      bcomp[i].setSelected(bvalues[i]);
    }

    int[] ivalues = {config.getInt("samplefreq"),
                     config.getInt("effect"),
		     config.getInt("ctrlr")};

    JComboBox[] icomp = {samplefreqBox,
                         effectBox,
		         ctrlrBox};

    for (int i = 0; i < ivalues.length; i++) {
      icomp[i].setSelectedIndex(ivalues[i]);
    }
    
    gammaBox.setSelectedIndex(config.getInt("gamma"));

    int[] svalues = {config.getInt("volume"),
                     config.getInt("brightness"),
		     config.getInt("beam"),
		     config.getInt("flicker"),
		     config.getInt("intensity")};

    JSlider[] scomp = {volumeBox, brightnessBox, beamBox,
                       flickerBox, intensityBox};

    for (int i = 0; i < svalues.length; i++) {
      scomp[i].setValue(svalues[i]);
    }

    String[] tvalues = {config.get("frameskip"),
                        config.get("artworkresolution")};

    JTextField[] tcomp = {frameskipField, aresolutionField};

    for (int i = 0; i < tvalues.length; i++) {
      tcomp[i].setText(tvalues[i]);
    }
  }

  /**
   * This method loads XMame options.
   *
   * @since  0.1.3pre3
   */
  public void loadLinuxOptions() {
    File gameConfig = new File(System.getProperty("user.home") + File.separator +
                               ".romeo4xmame" + File.separator +
                               "Games" + File.separator +
                               romeo.games.gameName + "rc");

    Config config = new Config(gameConfig);

    boolean[] bvalues = {config.getBoolean("timer"),
                         config.getBoolean("sound"),
			 config.getBoolean("samples"),
			 config.getBoolean("fakesound"),
                         config.getBoolean("autoframeskip"),
			 config.getBoolean("autodouble"),
			 config.getBoolean("window"),
			 config.getBoolean("throttle"),
			 config.getBoolean("scanlines"),
			 config.getBoolean("artwork"),
			 config.getBoolean("backdrop"),
			 config.getBoolean("overlay"),
			 config.getBoolean("bezel"),
			 config.getBoolean("crop"),
			 config.getBoolean("rotate"),
			 config.getBoolean("autoror"),
			 config.getBoolean("autorol"),
			 config.getBoolean("ror"),
			 config.getBoolean("rol"),
			 config.getBoolean("flipx"),
			 config.getBoolean("flipy"),
			 config.getBoolean("mouse"),
			 config.getBoolean("analogstick"),
			 config.getBoolean("hotrod"),
			 config.getBoolean("hotrodse"),
			 config.getBoolean("usbpspad"),
			 config.getBoolean("rapidfire"),
			 config.getBoolean("antialias"),
			 config.getBoolean("translucency")};

    JCheckBox[] bcomp = {timerBox, soundBox,
                         samplesBox, fakesoundBox,
                         autoframeskipBox, autodoubleBox,
			 windowBox, throttleBox,
			 scanlinesBox, artworkBox,
			 backdropBox, overlayBox,
			 bezelBox, cropBox,
			 rotateBox, autororBox,
			 autorolBox,rorBox,
			 rolBox, flipxBox,
			 flipyBox, mouseBox,
			 analogstickBox, hotrodBox,
			 hotrodseBox, usbpspadBox,
			 rapidfireBox, antialiasBox,
			 tlucencyBox};

    for (int i = 0; i < bvalues.length; i++) {
      bcomp[i].setSelected(bvalues[i]);
    }

    int[] ivalues = {config.getInt("dsp"),
                     config.getInt("mixer"),
		     config.getInt("samplefreq"),
		     config.getInt("frameskipper"),
                     config.getInt("effect"),
		     config.getInt("joytype")};

    JComboBox[] icomp = {dspBox, mixerBox, samplefreqBox, frameskipTypeBox,
                         effectBox, joytypeBox};

    for (int i = 0; i < ivalues.length; i++) {
      icomp[i].setSelectedIndex(ivalues[i]);
    }
    
    gammaBox.setSelectedIndex(config.getInt("gamma"));

    int[] svalues = {config.getInt("volume"),
                     config.getInt("brightness"), 
		     config.getInt("beam"),
		     config.getInt("flicker"), 
		     config.getInt("intensity")};

    JSlider[] scomp = {volumeBox, brightnessBox, beamBox,
                       flickerBox, intensityBox};

    for (int i = 0; i < svalues.length; i++) {
      scomp[i].setValue(svalues[i]);
    }

    String[] tvalues = {config.get("frameskip"),
                        config.get("bufsize"),
			config.get("audiodevice"),
			config.get("mixerdevice"),
                        config.get("artworkresolution"), 
			config.get("geometry"),
			config.get("joydevname")};

    JTextField[] tcomp = {frameskipField, bufsizeField,
                          audiodeviceField, mixerdeviceField,
			  aresolutionField, geometryField,
                          joydevnameField};

    for (int i = 0; i < tvalues.length; i++) {
      tcomp[i].setText(tvalues[i]);
    }
  }

  /**
   * SaveYourOptions Action
   *
   * @see #save()
   */
  class SaveYourOptionsAction extends AbstractAction {
    Options options;
    protected SaveYourOptionsAction(Options options) {
      super("SaveYourOptions");
      this.options = options;
    }

    public void actionPerformed(ActionEvent e) {
      save();
    }
  }

  /**
   * This method saves user options.
   *
   * @see   #saveWindowsOptions()
   * @see   #saveLinuxOptions()
   *
   * @since 0.1.3pre3
   */
  public void save() {
    String OS = System.getProperty("os.name").toLowerCase();
    // System.out.println(OS);
    if (OS.indexOf("windows") > -1) {
      saveWindowsOptions();
    }
    else {
      saveLinuxOptions();
    }
  }

  /**
   * This method saves Windows options.
   *
   * @since  0.1.3pre3
   */
  public void saveWindowsOptions() {
    File gameConfig = new File(System.getProperty("user.home") + File.separator +
                               ".romeo4xmame" + File.separator +
                               "Games" + File.separator +
                               romeo.games.gameName + "rc");

    Config config = new Config(gameConfig);

    String[] bparams = {"sound", "samples", "resamplefilter",
                        "autoframeskip", "window", "throttle",
			"scanlines", "artwork", "backdrop", "overlay",
			"bezel", "crop", "rotate", "autoror", "autorol",
			"ror", "rol", "flipx", "flipy", "mouse", "joystick",
			"steadykey", "keyboardleds", "antialias",
			"translucency"};

    boolean[] bvalues = {soundBox.isSelected(), samplesBox.isSelected(),
                         resamplefilterBox.isSelected(), autoframeskipBox.isSelected(),
			 windowBox.isSelected(), throttleBox.isSelected(),
			 scanlinesBox.isSelected(), artworkBox.isSelected(),
			 backdropBox.isSelected(), overlayBox.isSelected(),
			 bezelBox.isSelected(), cropBox.isSelected(),
			 rotateBox.isSelected(), autororBox.isSelected(),
			 autorolBox.isSelected(),rorBox.isSelected(),
			 rolBox.isSelected(), flipxBox.isSelected(),
			 flipyBox.isSelected(), mouseBox.isSelected(),
			 joystickBox.isSelected(), steadykeyBox.isSelected(),
			 keyboardledsBox.isSelected(), antialiasBox.isSelected(),
			 tlucencyBox.isSelected()};

    for (int i = 0; i < bparams.length; i++) {
      config.add(bparams[i], bvalues[i]);
    }

    String[] iparams = {"samplefreq", "effect", "gamma", "ctrlr"};

    int[] ivalues = {samplefreqBox.getSelectedIndex(),
                     effectBox.getSelectedIndex(),
		     gammaBox.getSelectedIndex(),
		     ctrlrBox.getSelectedIndex()};
		     
    for (int i = 0; i < iparams.length; i++) {
      config.add(iparams[i], ivalues[i]);
    }
    
    String[] sparams = {"volume", "brightness", "beam", "flicker", "intensity"};

    int[] svalues = {volumeBox.getValue(), brightnessBox.getValue(), beamBox.getValue(),
                     flickerBox.getValue(), intensityBox.getValue()};

    for (int i = 0; i < sparams.length; i++) {
      config.add(sparams[i], svalues[i]);
    }

    String[] tparams = {"frameskip", "artworkresolution"};

    String[] tvalues = {frameskipField.getText(), aresolutionField.getText()};

    for (int i = 0; i < tparams.length; i++) {
      config.add(tparams[i], tvalues[i]);
    }

    config.save();
  }

  /**
   * This method saves XMame options.
   *
   * @since  0.1.3pre3
   */
  public void saveLinuxOptions() {
    File gameConfig = new File(System.getProperty("user.home") + File.separator +
                               ".romeo4xmame" + File.separator +
                               "Games" + File.separator +
                               romeo.games.gameName + "rc");

    Config config = new Config(gameConfig);

    String[] bparams = {"timer", "sound", "samples", "fakesound",
                        "autoframeskip", "autodouble", "window", "throttle",
			"scanlines", "artwork", "backdrop", "overlay",
			"bezel", "crop", "rotate", "autoror", "autorol",
			"ror", "rol", "flipx", "flipy", "mouse", "analogstick",
			"hotrod", "hotrodse", "usbpspad", "rapidfire", "antialias",
			"translucency"};

    boolean[] bvalues = {timerBox.isSelected(), soundBox.isSelected(),
                         samplesBox.isSelected(), fakesoundBox.isSelected(),
                         autoframeskipBox.isSelected(), autodoubleBox.isSelected(),
			 windowBox.isSelected(), throttleBox.isSelected(),
			 scanlinesBox.isSelected(), artworkBox.isSelected(),
			 backdropBox.isSelected(), overlayBox.isSelected(),
			 bezelBox.isSelected(), cropBox.isSelected(),
			 rotateBox.isSelected(), autororBox.isSelected(),
			 autorolBox.isSelected(),rorBox.isSelected(),
			 rolBox.isSelected(), flipxBox.isSelected(),
			 flipyBox.isSelected(), mouseBox.isSelected(),
			 analogstickBox.isSelected(), hotrodBox.isSelected(),
			 hotrodseBox.isSelected(), usbpspadBox.isSelected(),
			 rapidfireBox.isSelected(), antialiasBox.isSelected(),
			 tlucencyBox.isSelected()};

    for (int i = 0; i < bparams.length; i++) {
      config.add(bparams[i], bvalues[i]);
    }

    String[] iparams = {"dsp", "mixer", "samplefreq", "frameskipper",
                        "effect", "gamma", "joytype"};

    int[] ivalues = {dspBox.getSelectedIndex(),
                     mixerBox.getSelectedIndex(),
                     samplefreqBox.getSelectedIndex(),
		     frameskipTypeBox.getSelectedIndex(),
                     effectBox.getSelectedIndex(),
		     gammaBox.getSelectedIndex(),
		     joytypeBox.getSelectedIndex()};

    for (int i = 0; i < iparams.length; i++) {
      config.add(iparams[i], ivalues[i]);
    }

    String[] sparams = {"volume", "brightness", "beam", "flicker", "intensity"};

    int[] svalues = {volumeBox.getValue(), brightnessBox.getValue(), beamBox.getValue(),
                     flickerBox.getValue(), intensityBox.getValue()};

    for (int i = 0; i < sparams.length; i++) {
      config.add(sparams[i], svalues[i]);
    }
    
    String[] tparams = {"frameskip", "bufsize", "audiodevice", "mixerdevice",
                        "artworkresolution", "geometry", "joydevname"};

    String[] tvalues = {frameskipField.getText(), bufsizeField.getText(),
                        audiodeviceField.getText(), mixerdeviceField.getText(),
			aresolutionField.getText(), geometryField.getText(),
                        joydevnameField.getText()};

    for (int i = 0; i < tparams.length; i++) {
      config.add(tparams[i], tvalues[i]);
    }
    
    config.save();
  }
  
  /**
   * This method returns Windows options.
   *
   * @return String Windows options
   *
   * @since  0.1.3pre3
   */
  public String getWindowsOptions() {
    String options = sound + samples + samplefreq + volume + resamplefilter +
		     autoframeskip + frameskip + window + throttle + scanlines + 
		     effect + gamma + brightness + artwork + backdrop + overlay +
		     bezel + artworkresolution + crop + rotate + autoror +
		     autorol + ror + rol + flipx + flipy + joystick + steadykey +
		     keyboardleds + mouse + ctrlr + beam + flicker + intensity +
		     antialias + translucency;

    return options;
  }
  
  /**
   * This method returns XMame options.
   *
   * @return String XMame options
   *
   * @since  0.1.3pre3
   */
  public String getLinuxOptions() {
    String options = dsp + timer + mixer + sound + samples + fakesound +
                     samplefreq + bufsize + volume + audiodevice + mixerdevice +
		     autoframeskip + frameskipType + frameskip + window +
		     throttle + scanlines + effect + gamma + brightness +
		     geometry + autodouble + artwork + backdrop + overlay +
		     bezel + artworkresolution + crop + rotate + autoror +
		     autorol + ror + rol + flipx + flipy + joystick + joytype +
		     joydevname + mouse + analogstick + hotrod + hotrodse +
		     usbpspad + rapidfire + beam + flicker + intensity +
		     antialias + translucency;

    return options;
  }
  
  /**
   * This method returns other useful options.
   *
   * @return String Additional options
   *
   * @since  0.1.3pre3
   */
  public String getOtherOptions() {
    String otherOptions = "";

    String[] params = {" -cfg_directory ", " -ctrlr_directory ",
                       " -diff_directory ", " -inipath ",
		       " -input_directory ", " -memcard_directory ",
		       " -nvram_directory ", " -state_directory ",
		       " -artwork_directory ", " -snapshot_directory ",
		       " -samplepath ", " -hiscore_file ",
		       " -history_file ", " -mameinfo_file ",
		       " -cheat_file "};

    config = new Config(new File(System.getProperty("user.home") +
                                 File.separator + ".romeo4xmame" +
				 File.separator + "Romeo.ini"));
    
    String[] values = {config.get("xmame.cfg_dir"),
                       config.get("xmame.ctrlr_dir"),
		       config.get("xmame.diff_dir"),
		       config.get("xmame.ini_dir"),
		       config.get("xmame.input_dir"),
		       config.get("xmame.memcard_dir"),
		       config.get("xmame.nvram_dir"),
		       config.get("xmame.state_dir"),
		       config.get("xmame.artwork_dir"),
		       config.get("xmame.snap_dir"),
		       config.get("xmame.sample_dir"),
		       config.get("xmame.hiscore.dat_path"),
		       config.get("xmame.history.dat_path"),
		       config.get("xmame.mameinfo.dat_path"),
		       config.get("xmame.cheat.dat_path")};

    for (int i = 0; i < values.length; i++) {
      if (values[i].length() > 0)
        otherOptions = otherOptions + params[i] + values[i];
      else
        otherOptions = otherOptions;
    }
    
    return otherOptions;
  }
} // End Options.java
