package com.miami.romeo.main.list;

import java.awt.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
// Jlist
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.DefaultListModel;

/**
 * Cell renderer for a JList
 */
public class InfoListCellRenderer extends JLabel implements ListCellRenderer {
  protected Border noFocusBorder;
  protected FontMetrics fm = null;
  protected Insets insets = new Insets(0, 0, 0, 0);

  protected int defaultTab = 50;
  protected int[] tabs = null;

  public InfoListCellRenderer () {
    super();
    noFocusBorder = new EmptyBorder(1, 2, 1, 2);
    setOpaque(true);
    setBorder(noFocusBorder);
  }

  // Set the attributes of the class and return a reference
  public Component getListCellRendererComponent(
    JList list,
    Object value,            // value to display
    int index,               // cell index
    boolean isSelected,      // is the cell selected
    boolean cellHasFocus)    // the list and the cell have the focus
    {
    setText(value.toString());

    setBackground(isSelected ? list.getSelectionBackground() : list.getBackground());
    setForeground(isSelected ? list.getSelectionForeground() : list.getForeground());

    setFont(list.getFont());
    setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

    return this;
  }

  public void setDefaultTab(int defaultTab) {
    defaultTab = defaultTab;
  }

  public int getDefaultTab() {
    return defaultTab;
  }

  public void setTabs(int[] tabs) {
    tabs = tabs;
  }

  public int[] getTabs() {
    return tabs;
  }

  public int getTab(int index) {
    if (tabs == null)
      return defaultTab*index;

    int len = tabs.length;
    if (index >= 0 && index < len)
      return tabs[index];

    return tabs[len - 1] + defaultTab * (index - len + 1);
  }

  public void paint(Graphics g) {
    fm = g.getFontMetrics();

    g.setColor(getBackground());
    g.fillRect(0, 0, getWidth(), getHeight());
    getBorder().paintBorder(this, g, 0, 0, getWidth(), getHeight());

    g.setColor(getForeground());
    g.setFont(getFont());
    insets = getInsets();
    int x = insets.left;
    int y = insets.top + fm.getAscent();

    StringTokenizer st = new StringTokenizer(getText(), "\t");
    while (st.hasMoreTokens()) {
      String sNext = st.nextToken();
      g.drawString(sNext, x, y);
      x += fm.stringWidth(sNext);

      if (!st.hasMoreTokens())
        break;
      int index = 0;
      while (x >= getTab(index))
	index++;
      x = getTab(index);
    }
  }
}
